@@ -128,7 +128,7 @@ static void wipe_schema(void);
 static void drop_dupes(const char *user, const char *host, const char *t);
 
 /**
- *  swing your pants 
+ *  swing your pants
  */
 int
 main(int argc, char *argv[])
@@ -255,10 +255,10 @@ main(int argc, char *argv[])
 	if(flag.import)
 	{
 		if(count.error && flag.verbose)
-			fprintf(stderr, "* I was unable to locate %i config files to import.\n",
+			fprintf(stderr, "* I was unable to locate %u config files to import.\n",
 				count.error);
 
-		fprintf(stdout, "* Import Stats: Klines: %i, Dlines: %i, Xlines: %i, Resvs: %i \n",
+		fprintf(stdout, "* Import Stats: Klines: %u, Dlines: %u, Xlines: %u, Resvs: %u \n",
 			count.klines, count.dlines, count.xlines, count.resvs);
 
 		fprintf(stdout,
@@ -542,6 +542,8 @@ import_config(const char *conf, int id)
 	if(flag.verbose)
 		fprintf(stdout, "%*s\n", strlen(bandb_suffix[id]) > 0 ? 10 : 15, "imported.");
 
+	fclose(fd);
+
 	return;
 }
 
