@@ -1,4 +1,9 @@
-/* $Id: ip_cloaking.c 3526 2007-07-06 07:56:14Z nenolod $ */
+/*
+ * Charybdis: an advanced ircd
+ * ip_cloaking.c: provide user hostname cloaking
+ *
+ * Written originally by nenolod, altered to use FNV by Elizabeth in 2008
+ */
 
 #include "stdinc.h"
 #include "modules.h"
@@ -6,19 +11,17 @@
 #include "client.h"
 #include "ircd.h"
 #include "send.h"
+#include "hash.h"
 #include "s_conf.h"
 #include "s_user.h"
 #include "s_serv.h"
 #include "numeric.h"
 
-/* if you're modifying this module, you'll probably to change this */
-#define KEY 0x13748cfa
-
 static int
 _modinit(void)
 {
 	/* add the usermode to the available slot */
-	user_modes['h'] = find_umode_slot();
+	user_modes['x'] = find_umode_slot();
 	construct_umodebuf();
 
 	return 0;
@@ -28,7 +31,7 @@ static void
 _moddeinit(void)
 {
 	/* disable the umode and remove it from the available list */
-	user_modes['h'] = 0;
+	user_modes['x'] = 0;
 	construct_umodebuf();
 }
 
@@ -68,67 +71,54 @@ distribute_hostchange(struct Client *client_p, char *newhost)
 		ClearDynSpoof(client_p);
 }
 
-#define Nval 0x8c3a48ac
-#define HOSTLEN 63
-#define INITDATA "98fwqefnoiqefv03f423t34gbv3vb89tg432t3b8" /* change this */
-
-static inline unsigned int
-get_string_entropy(const char *inbuf)
-{
-	unsigned int accum = 1;
-
-	while(*inbuf != '\0')
-		accum += *inbuf++;
-
-	return accum;
-}
-
-/* calls get_string_entropy() and toasts it against INITDATA */
-static inline unsigned int
-get_string_weighted_entropy(const char *inbuf)
-{
-	static int base_entropy = 0;
-        unsigned int accum = get_string_entropy(inbuf);
-
-	/* initialize the algorithm if it is not yet ready */
-	if (base_entropy == 0)
-		base_entropy = get_string_entropy(INITDATA);
-
-        return (Nval * accum) ^ base_entropy;
-}
-
 static void
 do_host_cloak_ip(const char *inbuf, char *outbuf)
 {
+	/* None of the characters in this table can be valid in an IP */
+	char chartable[] = "ghijklmnopqrstuvwxyz";
 	char *tptr;
-	unsigned int accum = get_string_weighted_entropy(inbuf);
-	char buf[HOSTLEN];
+	uint32_t accum = fnv_hash((const unsigned char*) inbuf, 32);
+	int sepcount = 0;
+	int totalcount = 0;
 	int ipv6 = 0;
 
-	strncpy(buf, inbuf, HOSTLEN);
-	tptr = strrchr(buf, '.');
+	rb_strlcpy(outbuf, inbuf, HOSTLEN + 1);
 
-	if (tptr == NULL)
+	if (strchr(outbuf, ':'))
 	{
-		tptr = strrchr(buf, ':');
 		ipv6 = 1;
-	}
 
-	if (tptr == NULL)
-	{
-		strncpy(outbuf, inbuf, HOSTLEN);
+		/* Damn you IPv6...
+		 * We count the number of colons so we can calculate how much
+		 * of the host to cloak. This is because some hostmasks may not
+		 * have as many octets as we'd like.
+		 *
+		 * We have to do this ahead of time because doing this during
+		 * the actual cloaking would get ugly
+		 */
+		for (tptr = outbuf; *tptr != '\0'; tptr++)
+			if (*tptr == ':')
+				totalcount++;
+	}
+	else if (!strchr(outbuf, '.'))
 		return;
-	}
 
-	*tptr++ = '\0';
+	for (tptr = outbuf; *tptr != '\0'; tptr++)
+	{
+		if (*tptr == ':' || *tptr == '.')
+		{
+			sepcount++;
+			continue;
+		}
 
-	if(ipv6)
-	{
-	    rb_snprintf(outbuf, HOSTLEN, "%s:%x", buf, accum);
-	}
-	else
-	{
-	    rb_snprintf(outbuf, HOSTLEN, "%s.%x", buf, accum);
+		if (ipv6 && sepcount < totalcount / 2)
+			continue;
+
+		if (!ipv6 && sepcount < 2)
+			continue;
+
+		*tptr = chartable[(*tptr + accum) % 20];
+		accum = (accum << 1) | (accum >> 31);
 	}
 }
 
@@ -137,12 +127,12 @@ do_host_cloak_host(const char *inbuf, char *outbuf)
 {
 	char b26_alphabet[] = "abcdefghijklmnopqrstuvwxyz";
 	char *tptr;
-	unsigned int accum = get_string_weighted_entropy(inbuf);
+	uint32_t accum = fnv_hash((const unsigned char*) inbuf, 32);
 
-	strncpy(outbuf, inbuf, HOSTLEN);
+	rb_strlcpy(outbuf, inbuf, HOSTLEN + 1);
 
-	/* pass 1: scramble first section of hostname using base26 
-	 * alphabet toasted against the weighted entropy of the string.
+	/* pass 1: scramble first section of hostname using base26
+	 * alphabet toasted against the FNV hash of the string.
 	 *
 	 * numbers are not changed at this time, only letters.
 	 */
@@ -151,20 +141,23 @@ do_host_cloak_host(const char *inbuf, char *outbuf)
 		if (*tptr == '.')
 			break;
 
-		if (isdigit(*tptr) || *tptr == '-')
+		if (isdigit((unsigned char)*tptr) || *tptr == '-')
 			continue;
 
-		*tptr = b26_alphabet[(*tptr * accum) % 26];
+		*tptr = b26_alphabet[(*tptr + accum) % 26];
+
+		/* Rotate one bit to avoid all digits being turned odd or even */
+		accum = (accum << 1) | (accum >> 31);
 	}
 
 	/* pass 2: scramble each number in the address */
 	for (tptr = outbuf; *tptr != '\0'; tptr++)
 	{
-		if (isdigit(*tptr))
-		{
-			*tptr = 48 + ((*tptr * accum) % 10);
-		}
-	}	
+		if (isdigit((unsigned char)*tptr))
+			*tptr = '0' + (*tptr + accum) % 10;
+
+		accum = (accum << 1) | (accum >> 31);
+	}
 }
 
 static void
@@ -177,14 +170,14 @@ check_umode_change(void *vdata)
 		return;
 
 	/* didn't change +h umode, we don't need to do anything */
-	if (!((data->oldumodes ^ source_p->umodes) & user_modes['h']))
+	if (!((data->oldumodes ^ source_p->umodes) & user_modes['x']))
 		return;
 
-	if (source_p->umodes & user_modes['h'])
+	if (source_p->umodes & user_modes['x'])
 	{
 		if (IsIPSpoof(source_p) || source_p->localClient->mangledhost == NULL || (IsDynSpoof(source_p) && strcmp(source_p->host, source_p->localClient->mangledhost)))
 		{
-			source_p->umodes &= ~user_modes['h'];
+			source_p->umodes &= ~user_modes['x'];
 			return;
 		}
 		if (strcmp(source_p->host, source_p->localClient->mangledhost))
@@ -195,7 +188,7 @@ check_umode_change(void *vdata)
 			sendto_one_numeric(source_p, RPL_HOSTHIDDEN, "%s :is now your hidden host",
 				source_p->host);
 	}
-	else if (!(source_p->umodes & user_modes['h']))
+	else if (!(source_p->umodes & user_modes['x']))
 	{
 		if (source_p->localClient->mangledhost != NULL &&
 				!strcmp(source_p->host, source_p->localClient->mangledhost))
@@ -212,20 +205,21 @@ check_new_user(void *vdata)
 
 	if (IsIPSpoof(source_p))
 	{
-		source_p->umodes &= ~user_modes['h'];
+		source_p->umodes &= ~user_modes['x'];
 		return;
 	}
-	source_p->localClient->mangledhost = rb_malloc(HOSTLEN);
+	source_p->localClient->mangledhost = rb_malloc(HOSTLEN + 1);
 	if (!irccmp(source_p->orighost, source_p->sockhost))
 		do_host_cloak_ip(source_p->orighost, source_p->localClient->mangledhost);
 	else
 		do_host_cloak_host(source_p->orighost, source_p->localClient->mangledhost);
 	if (IsDynSpoof(source_p))
-		source_p->umodes &= ~user_modes['h'];
-	if (source_p->umodes & user_modes['h'])
+		source_p->umodes &= ~user_modes['x'];
+	if (source_p->umodes & user_modes['x'])
 	{
 		rb_strlcpy(source_p->host, source_p->localClient->mangledhost, sizeof(source_p->host));
 		if (irccmp(source_p->host, source_p->orighost))
 			SetDynSpoof(source_p);
 	}
 }
+
