@@ -10,6 +10,8 @@
 #include "numeric.h"
 #include "s_conf.h"
 #include "modules.h"
+#include "messages.h"
+#include "send.h"
 
 #include <string.h>
 
@@ -46,6 +48,7 @@ m_mkpasswd(struct Client *client_p, struct Client *source_p, int parc, const cha
 {
 	static time_t last_used = 0;
 	char *salt;
+	const char *crypted;
 	const char *hashtype;
 	const char hashdefault[] = "SHA512";
 
@@ -82,7 +85,8 @@ m_mkpasswd(struct Client *client_p, struct Client *source_p, int parc, const cha
 		return 0;
 	}
 
-	sendto_one_notice(source_p, ":Hash [%s] for %s: %s", hashtype, parv[1], rb_crypt(parv[1], salt));
+	crypted = rb_crypt(parv[1], salt);
+	sendto_one_notice(source_p, ":Hash [%s] for %s: %s", hashtype, parv[1], crypted ? crypted : "???");
 	return 0;
 }
 
@@ -94,6 +98,7 @@ static int
 mo_mkpasswd(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
 {
 	char *salt;
+	const char *crypted;
 	const char *hashtype;
 	const char hashdefault[] = "SHA512";
 
@@ -121,7 +126,8 @@ mo_mkpasswd(struct Client *client_p, struct Client *source_p, int parc, const ch
 		return 0;
 	}
 
-	sendto_one_notice(source_p, ":Hash [%s] for %s: %s", hashtype, parv[1], rb_crypt(parv[1], salt));
+	crypted = rb_crypt(parv[1], salt);
+	sendto_one_notice(source_p, ":Hash [%s] for %s: %s", hashtype, parv[1], crypted ? crypted : "???");
 	return 0;
 }
 
@@ -183,34 +189,31 @@ char *
 generate_poor_salt(char *salt, int length)
 {
 	int i;
+
 	srand(time(NULL));
 	for(i = 0; i < length; i++)
-	{
 		salt[i] = saltChars[rand() % 64];
-	}
+
 	return (salt);
 }
 
 char *
 generate_random_salt(char *salt, int length)
 {
-	char *buf;
 	int fd, i;
-	if((fd = open("/dev/random", O_RDONLY)) < 0)
-	{
+
+	if((fd = open("/dev/urandom", O_RDONLY)) < 0)
 		return (generate_poor_salt(salt, length));
-	}
-	buf = calloc(1, length);
-	if(read(fd, buf, length) != length)
+
+	if(read(fd, salt, (size_t)length) != length)
 	{
-		free(buf);
+		close(fd);
 		return (generate_poor_salt(salt, length));
 	}
 
 	for(i = 0; i < length; i++)
-	{
-		salt[i] = saltChars[abs(buf[i]) % 64];
-	}
-	free(buf);
+		salt[i] = saltChars[abs(salt[i]) % 64];
+
+	close(fd);
 	return (salt);
 }
