@@ -76,6 +76,11 @@ struct Channel
 	unsigned long bants;
 	time_t channelts;
 	char *chname;
+
+	struct Client *last_checked_client;
+	time_t last_checked_ts;
+	unsigned int last_checked_type;
+	int last_checked_result;
 };
 
 struct membership
@@ -113,17 +118,7 @@ struct ChModeChange
 	const char *arg;
 	const char *id;
 	int dir;
-	int caps;
-	int nocaps;
 	int mems;
-	struct Client *client;
-};
-
-struct ChCapCombo
-{
-	int count;
-	int cap_yes;
-	int cap_no;
 };
 
 typedef void (*ChannelModeFunc)(struct Client *source_p, struct Channel *chptr,
@@ -169,14 +164,12 @@ typedef int (*ExtbanFunc)(const char *data, struct Client *client_p,
 #define MODE_INVITEONLY 0x0010
 #define MODE_NOPRIVMSGS 0x0020
 #define MODE_REGONLY	0x0040
-#define MODE_NOCOLOR	0x0080
 #define MODE_EXLIMIT	0x0100  /* exempt from list limits, +b/+e/+I/+q */
 #define MODE_PERMANENT  0x0200  /* permanant channel, +P */
 #define MODE_OPMODERATE 0x0400  /* send rejected messages to ops */
 #define MODE_FREEINVITE 0x0800  /* allow free use of /invite */
 #define MODE_FREETARGET 0x1000  /* can be forwarded to without authorization */
 #define MODE_DISFORWARD 0x2000  /* disable channel forwarding */
-#define MODE_NOCTCP     0x8000  /* Block CTCPs directed to this channel */
 
 #define CHFL_BAN        0x10000000	/* ban channel flag */
 #define CHFL_EXCEPTION  0x20000000	/* exception to ban channel flag */
@@ -217,7 +210,7 @@ void free_ban(struct Ban *bptr);
 
 extern void destroy_channel(struct Channel *);
 
-extern int can_send(struct Channel *chptr, struct Client *who, 
+extern int can_send(struct Channel *chptr, struct Client *who,
 		    struct membership *);
 extern int flood_attack_channel(int p_or_n, struct Client *source_p,
 				struct Channel *chptr, char *chname);
@@ -282,7 +275,7 @@ extern int match_extban(const char *banstr, struct Client *client_p, struct Chan
 extern int valid_extban(const char *banstr, struct Client *client_p, struct Channel *chptr, long mode_type);
 const char * get_extban_string(void);
 
-extern int get_channel_access(struct Client *source_p, struct membership *msptr);
+extern int get_channel_access(struct Client *source_p, struct membership *msptr, int dir);
 
 extern void send_channel_join(struct Channel *chptr, struct Client *client_p);
 
