@@ -133,10 +133,10 @@ struct Client
 	/* client->name is the unique name for a client nick or host */
 	char name[HOSTLEN + 1];
 
-	/* 
-	 * client->username is the username from ident or the USER message, 
-	 * If the client is idented the USER message is ignored, otherwise 
-	 * the username part of the USER message is put here prefixed with a 
+	/*
+	 * client->username is the username from ident or the USER message,
+	 * If the client is idented the USER message is ignored, otherwise
+	 * the username part of the USER message is put here prefixed with a
 	 * tilde depending on the I:line, Once a client has registered, this
 	 * field should be considered read-only.
 	 */
@@ -177,13 +177,13 @@ struct LocalUser
 	 * (directly connected to *this* server with a socket.
 	 */
 	/* Anti flooding part, all because of lamers... */
-	time_t last_join_time;	/* when this client last 
+	time_t last_join_time;	/* when this client last
 				   joined a channel */
-	time_t last_leave_time;	/* when this client last 
+	time_t last_leave_time;	/* when this client last
 				 * left a channel */
-	int join_leave_count;	/* count of JOIN/LEAVE in less than 
+	int join_leave_count;	/* count of JOIN/LEAVE in less than
 				   MIN_JOIN_LEAVE_TIME seconds */
-	int oper_warn_count_down;	/* warn opers of this possible 
+	int oper_warn_count_down;	/* warn opers of this possible
 					   spambot every time this gets to 0 */
 	time_t last_caller_id_time;
 
@@ -197,10 +197,10 @@ struct LocalUser
 	 * we want to use unsigned int here so the sizes have a better chance of
 	 * staying the same on 64 bit machines. The current trend is to use
 	 * I32LP64, (32 bit ints, 64 bit longs and pointers) and since ircd
-	 * will NEVER run on an operating system where ints are less than 32 bits, 
+	 * will NEVER run on an operating system where ints are less than 32 bits,
 	 * it's a relatively safe bet to use ints. Since right shift operations are
-	 * performed on these, it's not safe to allow them to become negative, 
-	 * which is possible for long running server connections. Unsigned values 
+	 * performed on these, it's not safe to allow them to become negative,
+	 * which is possible for long running server connections. Unsigned values
 	 * generally overflow gracefully. --Bleep
 	 */
 	unsigned int sendM;	/* Statistics: protocol messages send */
@@ -228,6 +228,7 @@ struct LocalUser
 	char *opername; /* name of operator{} block being used or tried (challenge) */
 	char *challenge;
 	char *fullcaps;
+	char *cipher_string;
 
 	int caps;		/* capabilities bit-field */
 	rb_fde_t *F;		/* >= 0, for local clients */
@@ -239,6 +240,7 @@ struct LocalUser
 
 	time_t next_away;	/* Don't allow next away before... */
 	time_t last;
+	uint32_t connid;
 
 	/* clients allowed to talk through +g */
 	rb_dlink_list allow_list;
@@ -277,12 +279,17 @@ struct LocalUser
 
 	struct _ssl_ctl *ssl_ctl;		/* which ssl daemon we're associate with */
 	struct _ssl_ctl *z_ctl;			/* second ctl for ssl+zlib */
+	uint32_t zconnid;
 	uint32_t localflags;
 	struct ZipStats *zipstats;		/* zipstats */
 	uint16_t cork_count;			/* used for corking/uncorking connections */
 	struct ev_entry *event;			/* used for associated events */
 
 	struct PrivilegeSet *privset;		/* privset... */
+
+	char sasl_agent[IDLEN];
+	unsigned char sasl_out;
+	unsigned char sasl_complete;
 };
 
 struct PreClient
@@ -291,10 +298,6 @@ struct PreClient
 	char spoofuser[USERLEN + 1];
 	char spoofhost[HOSTLEN + 1];
 
-	char sasl_agent[IDLEN];
-	unsigned char sasl_out;
-	unsigned char sasl_complete;
-
 	rb_dlink_list dnsbl_queries; /* list of struct BlacklistClient * */
 	struct Blacklist *dnsbl_listed; /* first dnsbl where it's listed */
 
@@ -442,11 +445,15 @@ struct ListClient
                              UMODE_WALLOP | UMODE_LOCOPS)
 #define DEFAULT_OPER_SNOMASK SNO_GENERAL
 
-#define CLICAP_MULTI_PREFIX	0x0001
-#define CLICAP_SASL		0x0002
-#define CLICAP_ACCOUNT_NOTIFY	0x0004
-#define CLICAP_EXTENDED_JOIN	0x0008
-#define CLICAP_AWAY_NOTIFY	0x0010
+#define CLICAP_MULTI_PREFIX		0x0001
+#define CLICAP_SASL			0x0002
+#define CLICAP_ACCOUNT_NOTIFY		0x0004
+#define CLICAP_EXTENDED_JOIN		0x0008
+#define CLICAP_AWAY_NOTIFY		0x0010
+#define CLICAP_TLS			0x0020
+#define CLICAP_USERHOST_IN_NAMES	0x0040
+#define CLICAP_CAP_NOTIFY		0x0080
+#define CLICAP_CHGHOST			0x0100
 
 /*
  * flags macros.
@@ -574,7 +581,6 @@ extern int is_remote_connect(struct Client *);
 extern void init_client(void);
 extern struct Client *make_client(struct Client *from);
 extern void free_pre_client(struct Client *client);
-extern void free_client(struct Client *client);
 
 extern int exit_client(struct Client *, struct Client *, struct Client *, const char *);
 
@@ -585,6 +591,8 @@ extern void error_exit_client(struct Client *, int);
 extern void count_local_client_memory(size_t * count, size_t * memory);
 extern void count_remote_client_memory(size_t * count, size_t * memory);
 
+extern int clean_nick(const char *, int loc_client);
+
 extern struct Client *find_chasing(struct Client *, const char *, int *);
 extern struct Client *find_person(const char *);
 extern struct Client *find_named_person(const char *);
@@ -595,7 +603,8 @@ extern void del_all_accepts(struct Client *client_p);
 
 extern void dead_link(struct Client *client_p, int sendqex);
 extern int show_ip(struct Client *source_p, struct Client *target_p);
-extern int show_ip_conf(struct ConfItem *aconf, struct Client *target_p);
+extern int show_ip_conf(struct ConfItem *aconf, struct Client *source_p);
+extern int show_ip_whowas(struct Whowas *whowas, struct Client *source_p);
 
 extern void initUser(void);
 extern void free_user(struct User *, struct Client *);
