@@ -28,23 +28,18 @@
 #define INCLUDED_numeric_h
 
 #include "config.h"
-
-/*
- * form_str - return a format string for a message number
- * messages are defined below
- */
-extern const char *form_str(int);
+#include "messages.h"
 
 /*
  * Reserve numerics 000-099 for server-client connections where the client
  * is local to the server. If any server is passed a numeric in this range
  * from another server then it is remapped to 100-199. -avalon
  */
-#define RPL_WELCOME          001
-#define RPL_YOURHOST         002
-#define RPL_CREATED          003
-#define RPL_MYINFO           004
-#define RPL_ISUPPORT         005
+#define RPL_WELCOME          1
+#define RPL_YOURHOST         2
+#define RPL_CREATED          3
+#define RPL_MYINFO           4
+#define RPL_ISUPPORT         5
 
 #define RPL_SNOMASK	     8
 
@@ -78,7 +73,7 @@ extern const char *form_str(int);
 #define RPL_STATSYLINE       218
 #define RPL_ENDOFSTATS       219
 /* note ircu uses 217 for STATSPLINE frip. conflict
- * as RPL_STATSQLINE was used in old 2.8 for Q line 
+ * as RPL_STATSQLINE was used in old 2.8 for Q line
  * I'm going to steal 220 for now *sigh*
  * -Dianora
  */
@@ -150,6 +145,7 @@ extern const char *form_str(int);
 
 #define RPL_ENDOFWHOIS       318
 #define RPL_WHOISCHANNELS    319
+#define RPL_WHOISSPECIAL     320
 
 #define RPL_LISTSTART        321
 #define RPL_LIST             322
@@ -307,7 +303,7 @@ extern const char *form_str(int);
 
 #define ERR_USERNOTONSERV    504
 
-/* #define ERR_LAST_ERR_MSG 505 
+/* #define ERR_LAST_ERR_MSG 505
  * moved to 999
  */
 #define ERR_WRONGPONG	     513
@@ -316,8 +312,11 @@ extern const char *form_str(int);
 
 #define ERR_HELPNOTFOUND     524
 
+#define RPL_STARTTLS         670 /* ircv3.atheme.org tls-3.1 */
 #define RPL_WHOISSECURE      671 /* Unreal3.2 --nenolod */
 
+#define ERR_STARTTLS         691 /* ircv3.atheme.org tls-3.2 */
+
 #define RPL_MODLIST          702
 #define RPL_ENDOFMODLIST     703
 
@@ -368,6 +367,8 @@ extern const char *form_str(int);
 #define ERR_MLOCKRESTRICTED	742
 #define ERR_INVALIDBAN		743
 
+#define ERR_TOPICLOCK		744 /* inspircd */
+
 #define RPL_SCANMATCHED		750
 #define RPL_SCANUMODES		751
 
