@@ -141,8 +141,6 @@ struct config_file_entry
 	const char *dpath;	/* DPATH if set from command line */
 	const char *configfile;
 
-	char *egdpool_path;
-
 	char *default_operstring;
 	char *default_adminstring;
 	char *servicestring;
@@ -150,7 +148,9 @@ struct config_file_entry
 
 	char *identifyservice;
 	char *identifycommand;
-	
+
+	char *sasl_service;
+
 	char *fname_userlog;
 	char *fname_fuserlog;
 	char *fname_operlog;
@@ -206,7 +206,6 @@ struct config_file_entry
 	int min_nonwildcard_simple;
 	int default_floodcount;
 	int default_ident_timeout;
-	int use_egd;
 	int ping_cookie;
 	int tkline_expire_notices;
 	int use_whois_actually;
@@ -235,6 +234,7 @@ struct config_file_entry
 	int client_flood_message_num;
 
 	unsigned int nicklen;
+	int certfp_method;
 };
 
 struct config_channel_entry
@@ -258,6 +258,8 @@ struct config_channel_entry
 	int resv_forcepart;
 	int channel_target_change;
 	int disable_local_channels;
+	unsigned int autochanmodes;
+	int displayed_usercount;
 };
 
 struct config_server_hide
@@ -274,7 +276,6 @@ struct server_info
 	char sid[4];
 	char *description;
 	char *network_name;
-	char *network_desc;
 	int hub;
 	struct sockaddr_in ip;
 	int default_max_clients;
@@ -289,6 +290,7 @@ struct server_info
 	char *ssl_ca_cert;
 	char *ssl_cert;
 	char *ssl_dh_params;
+	char *ssl_cipher_list;
 	int ssld_count;
 };
 
@@ -339,7 +341,7 @@ extern struct ConfItem *make_conf(void);
 extern void free_conf(struct ConfItem *);
 
 extern rb_dlink_node *find_prop_ban(unsigned int status, const char *user, const char *host);
-extern void deactivate_conf(struct ConfItem *, rb_dlink_node *);
+extern void deactivate_conf(struct ConfItem *, rb_dlink_node *, time_t);
 extern void replace_old_ban(struct ConfItem *);
 
 extern void read_conf_files(int cold);
@@ -352,7 +354,7 @@ extern int detach_conf(struct Client *);
 extern struct ConfItem *find_tkline(const char *, const char *, struct sockaddr *);
 extern char *show_iline_prefix(struct Client *, struct ConfItem *, char *);
 extern void get_printable_conf(struct ConfItem *,
-			       char **, char **, char **, char **, int *, char **);
+			       char **, char **, const char **, char **, int *, char **);
 extern char *get_user_ban_reason(struct ConfItem *aconf);
 extern void get_printable_kline(struct Client *, struct ConfItem *,
 				char **, char **, char **, char **);
