@@ -68,19 +68,20 @@ extern void sendto_channel_local_with_capability(int type, int caps, int negcaps
 extern void sendto_channel_local_with_capability_butone(struct Client *, int type, int caps, int negcaps, struct Channel *,
 							const char *, ...) AFP(6, 7);
 
-extern void sendto_common_channels_local(struct Client *, int cap, const char *, ...) AFP(3, 4);
-extern void sendto_common_channels_local_butone(struct Client *, int cap, const char *, ...) AFP(3, 4);
+extern void sendto_common_channels_local(struct Client *, int cap, int negcap, const char *, ...) AFP(4, 5);
+extern void sendto_common_channels_local_butone(struct Client *, int cap, int negcap, const char *, ...) AFP(4, 5);
 
 
 extern void sendto_match_butone(struct Client *, struct Client *,
 				const char *, int, const char *, ...) AFP(5, 6);
-extern void sendto_match_servs(struct Client *source_p, const char *mask, 
+extern void sendto_match_servs(struct Client *source_p, const char *mask,
 				int capab, int, const char *, ...) AFP(5, 6);
 
 extern void sendto_monitor(struct monitor *monptr, const char *, ...) AFP(2, 3);
 
 extern void sendto_anywhere(struct Client *, struct Client *, const char *,
 			    const char *, ...) AFP(4, 5);
+extern void sendto_local_clients_with_capability(int cap, const char *pattern, ...) AFP(2, 3);
 
 extern void sendto_realops_snomask(int, int, const char *, ...) AFP(3, 4);
 extern void sendto_realops_snomask_from(int, int, struct Client *, const char *, ...) AFP(4, 5);
@@ -89,7 +90,7 @@ extern void sendto_wallops_flags(int, struct Client *, const char *, ...) AFP(3,
 
 extern void kill_client(struct Client *client_p, struct Client *diedie,
 			 const char *pattern, ...) AFP(3, 4);
-extern void kill_client_serv_butone(struct Client *one, struct Client *source_p, 
+extern void kill_client_serv_butone(struct Client *one, struct Client *source_p,
 				    const char *pattern, ...) AFP(3, 4);
 
 #define L_ALL 	0
