@@ -97,10 +97,37 @@ void rb_note(rb_fde_t *, const char *);
 #define	RB_SELECT_READ		0x1
 #define	RB_SELECT_WRITE		0x2
 
-#define RB_SELECT_ACCEPT		RB_SELECT_READ
-#define RB_SELECT_CONNECT		RB_SELECT_WRITE
+#define RB_SELECT_ACCEPT	RB_SELECT_READ
+#define RB_SELECT_CONNECT	RB_SELECT_WRITE
 
-#define RB_SSL_CERTFP_LEN 20
+#define RB_SSL_CERTFP_LEN	64
+
+/* Methods for certfp */
+/* Digest of full X.509 certificate */
+#define RB_SSL_CERTFP_METH_CERT_SHA1    0x0000
+#define RB_SSL_CERTFP_METH_CERT_SHA256  0x0001
+#define RB_SSL_CERTFP_METH_CERT_SHA512  0x0002
+/* Digest of SubjectPublicKeyInfo (RFC 5280), used by DANE (RFC 6698) */
+#define RB_SSL_CERTFP_METH_SPKI_SHA256  0x1001
+#define RB_SSL_CERTFP_METH_SPKI_SHA512  0x1002
+
+/* Names for certfp */
+#define CERTFP_NAME_CERT_SHA1           "sha1"
+#define CERTFP_PREFIX_CERT_SHA1         ""
+#define CERTFP_NAME_CERT_SHA256         "sha256"
+#define CERTFP_PREFIX_CERT_SHA256       ""
+#define CERTFP_NAME_CERT_SHA512         "sha512"
+#define CERTFP_PREFIX_CERT_SHA512       ""
+/* These prefixes are copied from RFC 7218 */
+#define CERTFP_NAME_SPKI_SHA256         "spki_sha256"
+#define CERTFP_PREFIX_SPKI_SHA256       "SPKI:SHA2-256:"
+#define CERTFP_NAME_SPKI_SHA512         "spki_sha512"
+#define CERTFP_PREFIX_SPKI_SHA512       "SPKI:SHA2-512:"
+
+
+#define RB_SSL_CERTFP_LEN_SHA1		20
+#define RB_SSL_CERTFP_LEN_SHA256	32
+#define RB_SSL_CERTFP_LEN_SHA512	64
 
 int rb_set_nb(rb_fde_t *);
 int rb_set_buffers(rb_fde_t *, int);
@@ -125,7 +152,7 @@ ssize_t rb_writev(rb_fde_t *, struct rb_iovec *vector, int count);
 ssize_t rb_read(rb_fde_t *, void *buf, int count);
 int rb_pipe(rb_fde_t **, rb_fde_t **, const char *desc);
 
-int rb_setup_ssl_server(const char *cert, const char *keyfile, const char *dhfile);
+int rb_setup_ssl_server(const char *cert, const char *keyfile, const char *dhfile, const char *cipher_list);
 int rb_ssl_listen(rb_fde_t *, int backlog, int defer_accept);
 int rb_listen(rb_fde_t *, int backlog, int defer_accept);
 
@@ -143,7 +170,7 @@ int rb_select(unsigned long);
 int rb_fd_ssl(rb_fde_t *F);
 int rb_get_fd(rb_fde_t *F);
 const char *rb_get_ssl_strerror(rb_fde_t *F);
-int rb_get_ssl_certfp(rb_fde_t *F, uint8_t certfp[RB_SSL_CERTFP_LEN]);
+int rb_get_ssl_certfp(rb_fde_t *F, uint8_t certfp[RB_SSL_CERTFP_LEN], int method);
 
 rb_fde_t *rb_get_fde(int fd);
 
@@ -157,7 +184,6 @@ const char *rb_get_iotype(void);
 
 typedef enum
 {
-	RB_PRNG_EGD,
 	RB_PRNG_FILE,
 #ifdef _WIN32
 	RB_PRNGWIN32,
@@ -167,7 +193,6 @@ typedef enum
 
 int rb_init_prng(const char *path, prng_seed_t seed_type);
 int rb_get_random(void *buf, size_t len);
-int rb_get_pseudo_random(void *buf, size_t len);
 void rb_ssl_start_accepted(rb_fde_t *new_F, ACCB * cb, void *data, int timeout);
 void rb_ssl_start_connected(rb_fde_t *F, CNCB * callback, void *data, int timeout);
 int rb_supports_ssl(void);
@@ -179,4 +204,6 @@ void rb_ssl_clear_handshake_count(rb_fde_t *F);
 int rb_pass_fd_to_process(rb_fde_t *, pid_t, rb_fde_t *);
 rb_fde_t *rb_recv_fd(rb_fde_t *);
 
+const char *rb_ssl_get_cipher(rb_fde_t *F);
+
 #endif /* INCLUDED_commio_h */
