@@ -59,7 +59,8 @@ static struct ev_entry *rb_timeout_ev;
 static const char *rb_err_str[] = { "Comm OK", "Error during bind()",
 	"Error during DNS lookup", "connect timeout",
 	"Error during connect()",
-	"Comm Error"
+	"Comm Error",
+	"Error with SSL"
 };
 
 /* Highest FD and number of open FDs .. */
@@ -198,7 +199,7 @@ rb_getmaxconnect(void)
 
 /*
  * set_sock_buffers - set send and receive buffers for socket
- * 
+ *
  * inputs	- fd file descriptor
  * 		- size to set
  * output       - returns true (1) if successful, false (0) otherwise
@@ -217,7 +218,7 @@ rb_set_buffers(rb_fde_t *F, int size)
 }
 
 /*
- * set_non_blocking - Set the client connection into non-blocking mode. 
+ * set_non_blocking - Set the client connection into non-blocking mode.
  *
  * inputs	- fd to set into non blocking mode
  * output	- 1 if successful 0 if not
@@ -695,7 +696,7 @@ rb_socket(int family, int sock_type, int proto, const char *note)
 		return NULL;	/* errno will be passed through, yay.. */
 
 #if defined(RB_IPV6) && defined(IPV6_V6ONLY)
-	/* 
+	/*
 	 * Make sure we can take both IPv4 and IPv6 connections
 	 * on an AF_INET6 socket
 	 */
@@ -771,7 +772,7 @@ rb_listen(rb_fde_t *F, int backlog, int defer_accept)
 #ifdef TCP_DEFER_ACCEPT
 	if (defer_accept && !result)
 	{
-		setsockopt(F->fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &backlog, sizeof(int));
+		(void)setsockopt(F->fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &backlog, sizeof(int));
 	}
 #endif
 #ifdef SO_ACCEPTFILTER
@@ -990,8 +991,8 @@ rb_read(rb_fde_t *F, void *buf, int count)
 	if(F == NULL)
 		return 0;
 
-	/* This needs to be *before* RB_FD_SOCKET otherwise you'll process 
-	 * an SSL socket as a regular socket 
+	/* This needs to be *before* RB_FD_SOCKET otherwise you'll process
+	 * an SSL socket as a regular socket
 	 */
 #ifdef HAVE_SSL
 	if(F->type & RB_FD_SSL)
@@ -1103,7 +1104,7 @@ rb_writev(rb_fde_t *F, struct rb_iovec * vector, int count)
 #endif
 
 
-/* 
+/*
  * From: Thomas Helvey <tomh@inxpress.net>
  */
 static const char *IpQuadTab[] = {
@@ -1513,7 +1514,7 @@ inet_pton6(const char *src, unsigned char *dst)
 	curtok = src;
 	saw_xdigit = 0;
 	val = 0;
-	while((ch = tolower(*src++)) != '\0')
+	while((ch = tolower((unsigned char)*src++)) != '\0')
 	{
 		const char *pch;
 
@@ -2214,14 +2215,12 @@ rb_recv_fd_buf(rb_fde_t *F, void *data, size_t datasize, rb_fde_t **xF, int nfds
 int
 rb_send_fd_buf(rb_fde_t *xF, rb_fde_t **F, int count, void *data, size_t datasize, pid_t pid)
 {
-	int n;
 	struct msghdr msg;
 	struct cmsghdr *cmsg;
 	struct iovec iov[1];
 	char empty = '0';
-	char *buf;
 
-	memset(&msg, 0, sizeof(&msg));
+	memset(&msg, 0, sizeof(msg));
 	if(datasize == 0)
 	{
 		iov[0].iov_base = &empty;
@@ -2242,9 +2241,8 @@ rb_send_fd_buf(rb_fde_t *xF, rb_fde_t **F, int count, void *data, size_t datasiz
 
 	if(count > 0)
 	{
-		int i;
 		int len = CMSG_SPACE(sizeof(int) * count);
-		buf = alloca(len);
+		char buf[len];
 
 		msg.msg_control = buf;
 		msg.msg_controllen = len;
@@ -2253,14 +2251,14 @@ rb_send_fd_buf(rb_fde_t *xF, rb_fde_t **F, int count, void *data, size_t datasiz
 		cmsg->cmsg_type = SCM_RIGHTS;
 		cmsg->cmsg_len = CMSG_LEN(sizeof(int) * count);
 
-		for(i = 0; i < count; i++)
+		for(unsigned int i = 0; i < count; i++)
 		{
 			((int *)CMSG_DATA(cmsg))[i] = rb_get_fd(F[i]);
 		}
 		msg.msg_controllen = cmsg->cmsg_len;
+		return sendmsg(rb_get_fd(xF), &msg, MSG_NOSIGNAL);
 	}
-	n = sendmsg(rb_get_fd(xF), &msg, MSG_NOSIGNAL);
-	return n;
+	return sendmsg(rb_get_fd(xF), &msg, MSG_NOSIGNAL);
 }
 #else
 #ifndef _WIN32
