@@ -405,7 +405,7 @@ rb_epoll_sched_event_signalfd(struct ev_entry *event, int when)
 	struct sigevent ev;
 	struct itimerspec ts;
 
-	memset(&ev, 0, sizeof(&ev));
+	memset(&ev, 0, sizeof(ev));
 	event->comm_ptr = rb_malloc(sizeof(timer_t));
 	id = event->comm_ptr;
 	ev.sigev_notify = SIGEV_SIGNAL;
