@@ -76,6 +76,8 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	struct Client *target_p;
 	int hop;
 	unsigned int required_mask;
+	const char *missing;
+	int ret;
 
 	name = parv[1];
 	hop = atoi(parv[2]);
@@ -92,7 +94,7 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		return 0;
 	}
 
-	/* 
+	/*
 	 * Reject a direct nonTS server connection if we're TS_ONLY -orabidoo
 	 */
 	if(!DoesTS(client_p))
@@ -109,20 +111,14 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		return 0;
 	}
 
-	/* check to ensure any "required" caps are set. --nenolod */
-	/* XXX: show required CAPABs. */
-	required_mask = capability_index_get_required(serv_capindex);
-	if (required_mask && !(client_p->localClient->caps & required_mask))
-	{
-		exit_client(client_p, client_p, client_p, "Missing required CAPABs");
-
-		return 0;
-	}
-
 	/* Now we just have to call check_server and everything should be
 	 * check for us... -A1kmm. */
-	switch (check_server(name, client_p))
+	ret = check_server(name, client_p);
+	switch (ret)
 	{
+	case 0:
+		/* success */
+		break;
 	case -1:
 		if(ConfigFileEntry.warn_no_nline)
 		{
@@ -187,11 +183,42 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		sendto_realops_snomask(SNO_GENERAL, L_ALL,
 		     "Connection from servername %s requires SSL/TLS but is plaintext",
 		     name);
-		ilog(L_SERVER, "Access denied, requires SSL/TLS but is plaintext from %s", 
+		ilog(L_SERVER, "Access denied, requires SSL/TLS but is plaintext from %s",
 		     log_client_name(client_p, SHOW_IP));
 
 		exit_client(client_p, client_p, client_p, "Access denied, requires SSL/TLS but is plaintext");
 		return 0;
+	case -6:
+		if (client_p->certfp)
+		{
+			sendto_realops_snomask(SNO_GENERAL, L_ALL,
+			     "Connection from servername %s has invalid certificate fingerprint %s",
+			     name, client_p->certfp);
+			ilog(L_SERVER, "Access denied, invalid certificate fingerprint %s from %s",
+			     client_p->certfp, log_client_name(client_p, SHOW_IP));
+			exit_client(client_p, client_p, client_p, "Invalid fingerprint.");
+		}
+		else
+		{
+			sendto_realops_snomask(SNO_GENERAL, L_ALL,
+			    "Connection from servername %s failed certificate validation",
+			    name);
+			ilog(L_SERVER, "Access denied; certificate validation failed for certificate from %s",
+			    log_client_name(client_p, SHOW_IP));
+			exit_client(client_p, client_p, client_p, "Invalid certificate.");
+		}
+
+		return 0;
+	default:
+		sendto_realops_snomask(SNO_GENERAL, L_ALL,
+		     "Connection from servername %s rejected, unknown error %d",
+		     name, ret);
+		ilog(L_SERVER, "Access denied, unknown error %d for server %s%s", ret,
+		     EmptyString(client_p->name) ? name : "",
+		     log_client_name(client_p, SHOW_IP));
+
+		exit_client(client_p, client_p, client_p, "Unknown error.");
+		return 0;
 	}
 
 	/* require TS6 for direct links */
@@ -203,6 +230,28 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		return 0;
 	}
 
+	/* check to ensure any "required" caps are set. --nenolod */
+	required_mask = capability_index_get_required(serv_capindex);
+	if (!IsCapable(client_p, required_mask))
+	{
+		missing = capability_index_list(serv_capindex, required_mask &
+				~client_p->localClient->caps);
+		sendto_realops_snomask(SNO_GENERAL, is_remote_connect(client_p) ? L_NETWIDE : L_ALL,
+					"Link %s dropped, required CAPABs [%s] are missing",
+					name, missing);
+		ilog(L_SERVER, "Link %s%s dropped, required CAPABs [%s] are missing",
+				EmptyString(client_p->name) ? name : "",
+				log_client_name(client_p, SHOW_IP), missing);
+		/* Do not use '[' in the below message because it would cause
+		 * it to be considered potentially unsafe (might disclose IP
+		 * addresses)
+		 */
+		sendto_one(client_p, "ERROR :Missing required CAPABs (%s)", missing);
+		exit_client(client_p, client_p, client_p, "Missing required CAPABs");
+
+		return 0;
+	}
+
 	if((target_p = find_server(NULL, name)))
 	{
 		/*
@@ -294,7 +343,7 @@ ms_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	hop = atoi(parv[2]);
 	rb_strlcpy(info, parv[3], sizeof(info));
 
-	if((target_p = find_server(NULL, name)))
+	if(find_server(NULL, name))
 	{
 		/*
 		 * This link is trying feed me a server that I already have
@@ -312,7 +361,7 @@ ms_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		 * that already exists, then sends you a client burst, you squit the
 		 * server, but you keep getting the burst of clients on a server that
 		 * doesnt exist, although ircd can handle it, its not a realistic
-		 * solution.. --fl_ 
+		 * solution.. --fl_
 		 */
 		ilog(L_SERVER, "Link %s cancelled, server %s already exists",
 			client_p->name, name);
@@ -324,7 +373,7 @@ ms_server(struct Client *client_p, struct Client *source_p, int parc, const char
 		return 0;
 	}
 
-	/* 
+	/*
 	 * User nicks never have '.' in them and server names
 	 * must always have '.' in them.
 	 */
@@ -380,7 +429,7 @@ ms_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	 *            name = "irc.bighub.net";
 	 *            hub_mask="*";
 	 *            ...
-	 * 
+	 *
 	 * That would allow "irc.bighub.net" to introduce anything it wanted..
 	 *
 	 * However
@@ -418,7 +467,7 @@ ms_server(struct Client *client_p, struct Client *source_p, int parc, const char
 				     "Link %s introduced leafed server %s.",
 				     client_p->name, name);
 		ilog(L_SERVER, "Link %s introduced leafed server %s.",
-			client_p->name, name);	
+			client_p->name, name);
 
 		rb_snprintf(squitreason, sizeof squitreason,
 				"Matching leaf_mask for %s",
@@ -490,7 +539,7 @@ ms_sid(struct Client *client_p, struct Client *source_p, int parc, const char *p
 	char squitreason[160];
 
 	/* collision on the name? */
-	if((target_p = find_server(NULL, parv[1])) != NULL)
+	if(find_server(NULL, parv[1]) != NULL)
 	{
 		ilog(L_SERVER, "Link %s cancelled, server %s already exists",
 			client_p->name, parv[1]);
@@ -588,7 +637,7 @@ ms_sid(struct Client *client_p, struct Client *source_p, int parc, const char *p
 				     "Link %s introduced leafed server %s.",
 				     client_p->name, parv[1]);
 		ilog(L_SERVER, "Link %s introduced leafed server %s.",
-			client_p->name, parv[1]);	
+			client_p->name, parv[1]);
 
 		rb_snprintf(squitreason, sizeof squitreason,
 				"Matching leaf_mask for %s",
