@@ -428,7 +428,7 @@ mo_unkline(struct Client *client_p, struct Client *source_p, int parc, const cha
 		sendto_one_notice(source_p, ":No K-Line for %s@%s", user, host);
 		return 0;
 	}
-	
+
 	if(aconf->lifetime)
 	{
 		if(propagated)
@@ -509,7 +509,7 @@ handle_remote_unkline(struct Client *source_p, const char *user, const char *hos
 
 /* apply_kline()
  *
- * inputs	- 
+ * inputs	-
  * output	- NONE
  * side effects	- kline as given, is added to the hashtable
  *		  and conf file
@@ -632,7 +632,7 @@ apply_prop_kline(struct Client *source_p, struct ConfItem *aconf,
 }
 
 /* find_user_host()
- * 
+ *
  * inputs	- client placing kline, user@host, user buffer, host buffer
  * output	- 0 if not ok to kline, 1 to kline i.e. if valid user host
  * side effects -
@@ -724,7 +724,7 @@ already_placed_kline(struct Client *source_p, const char *luser, const char *lho
 	if(aconf == NULL && ConfigFileEntry.non_redundant_klines)
 	{
 		bits = 0;
-		if((t = parse_netmask(lhost, (struct sockaddr *) &iphost, &bits)) != HM_HOST)
+		if((t = parse_netmask(lhost, &iphost, &bits)) != HM_HOST)
 		{
 #ifdef RB_IPV6
 			if(t == HM_IPV6)
@@ -834,6 +834,7 @@ static void
 remove_prop_kline(struct Client *source_p, struct ConfItem *aconf)
 {
 	rb_dlink_node *ptr;
+	time_t now;
 
 	ptr = rb_dlinkFind(aconf, &prop_bans);
 	if (!ptr)
@@ -848,8 +849,9 @@ remove_prop_kline(struct Client *source_p, struct ConfItem *aconf)
 
 	ilog(L_KLINE, "UK %s %s %s",
 	     get_oper_name(source_p), aconf->user, aconf->host);
-	if(aconf->created < rb_current_time())
-		aconf->created = rb_current_time();
+	now = rb_current_time();
+	if(aconf->created < now)
+		aconf->created = now;
 	else
 		aconf->created++;
 	aconf->hold = aconf->created;
@@ -863,5 +865,5 @@ remove_prop_kline(struct Client *source_p, struct ConfItem *aconf)
 			0,
 			(int)(aconf->lifetime - aconf->created));
 	remove_reject_mask(aconf->user, aconf->host);
-	deactivate_conf(aconf, ptr);
+	deactivate_conf(aconf, ptr, now);
 }
