@@ -48,6 +48,9 @@
 #include "msg.h"
 #include "parse.h"
 #include "modules.h"
+#include "inline/stringops.h"
+#include "s_assert.h"
+#include "logger.h"
 
 static rb_dlink_list safelisting_clients = { NULL, NULL, 0 };
 
@@ -57,6 +60,8 @@ static void _moddeinit(void);
 static int m_list(struct Client *, struct Client *, int, const char **);
 static int mo_list(struct Client *, struct Client *, int, const char **);
 
+static void list_one_channel(struct Client *source_p, struct Channel *chptr, int visible);
+
 static void safelist_check_cliexit(hook_data_client_exit * hdata);
 static void safelist_client_instantiate(struct Client *, struct ListClient *);
 static void safelist_client_release(struct Client *);
@@ -191,11 +196,10 @@ static int mo_list(struct Client *client_p, struct Client *source_p, int parc, c
 	/* Multiple channels, possibly with parameters. */
 	params = rb_malloc(sizeof(struct ListClient));
 
-	/* XXX rather arbitrary -- jilles */
-	params->users_min = 3;
+	params->users_min = ConfigChannel.displayed_usercount;
 	params->users_max = INT_MAX;
 	params->operspy = operspy;
-	params->created_min = params->topic_min = 
+	params->created_min = params->topic_min =
 		params->created_max = params->topic_max = 0;
 
 	if (args && !EmptyString(args))
@@ -285,6 +289,29 @@ static int mo_list(struct Client *client_p, struct Client *source_p, int parc, c
 	return 0;
 }
 
+/*
+ * list_one_channel()
+ *
+ * inputs       - client pointer, channel pointer, whether normally visible
+ * outputs      - none
+ * side effects - a channel is listed
+ */
+static void list_one_channel(struct Client *source_p, struct Channel *chptr,
+		int visible)
+{
+	char topic[TOPICLEN + 1];
+
+	if (chptr->topic != NULL)
+		rb_strlcpy(topic, chptr->topic, sizeof topic);
+	else
+		topic[0] = '\0';
+	strip_colour(topic);
+	sendto_one(source_p, form_str(RPL_LIST), me.name, source_p->name,
+		   visible ? "" : "!",
+		   chptr->chname, rb_dlink_list_length(&chptr->members),
+		   topic);
+}
+
 /*
  * safelist_sendq_exceeded()
  *
@@ -392,10 +419,7 @@ static void safelist_channel_named(struct Client *source_p, const char *name, in
 
 	visible = !SecretChannel(chptr) || IsMember(source_p, chptr);
 	if (visible || operspy)
-		sendto_one(source_p, form_str(RPL_LIST), me.name, source_p->name,
-			   visible ? "" : "!",
-			   chptr->chname, rb_dlink_list_length(&chptr->members),
-			   chptr->topic == NULL ? "" : chptr->topic);
+		list_one_channel(source_p, chptr, visible);
 
 	sendto_one(source_p, form_str(RPL_LISTEND), me.name, source_p->name);
 	return;
@@ -436,10 +460,7 @@ static void safelist_one_channel(struct Client *source_p, struct Channel *chptr)
 	if (safelist_data->created_max && chptr->channelts > safelist_data->created_max)
 		return;
 
-	sendto_one(source_p, form_str(RPL_LIST), me.name, source_p->name,
-		   visible ? "" : "!",
-		   chptr->chname, rb_dlink_list_length(&chptr->members),
-		   chptr->topic == NULL ? "" : chptr->topic);
+	list_one_channel(source_p, chptr, visible);
 }
 
 /*
@@ -462,7 +483,7 @@ static void safelist_iterate_client(struct Client *source_p)
 			return;
 		}
 
-		RB_DLINK_FOREACH(ptr, channelTable[iter].head) 
+		RB_DLINK_FOREACH(ptr, channelTable[iter].head)
 			safelist_one_channel(source_p, (struct Channel *) ptr->data);
 	}
 
@@ -473,6 +494,6 @@ static void safelist_iterate_clients(void *unused)
 {
 	rb_dlink_node *n, *n2;
 
-	RB_DLINK_FOREACH_SAFE(n, n2, safelisting_clients.head) 
+	RB_DLINK_FOREACH_SAFE(n, n2, safelisting_clients.head)
 		safelist_iterate_client((struct Client *)n->data);
 }
