@@ -73,7 +73,6 @@ static void remove_resv(struct Client *source_p, const char *name, int propagate
 
 /*
  * mo_resv()
- *      parv[0] = sender prefix
  *      parv[1] = channel/nick to forbid
  *      parv[2] = reason
  */
@@ -151,7 +150,6 @@ mo_resv(struct Client *client_p, struct Client *source_p, int parc, const char *
 }
 
 /* ms_resv()
- *     parv[0] = sender prefix
  *     parv[1] = target server
  *     parv[2] = channel/nick to forbid
  *     parv[3] = reason
@@ -422,7 +420,6 @@ cluster_resv(struct Client *source_p, int temp_time, const char *name, const cha
 
 /*
  * mo_unresv()
- *     parv[0] = sender prefix
  *     parv[1] = channel/nick to unforbid
  */
 static int
@@ -463,7 +460,6 @@ mo_unresv(struct Client *client_p, struct Client *source_p, int parc, const char
 }
 
 /* ms_unresv()
- *     parv[0] = sender prefix
  *     parv[1] = target server
  *     parv[2] = resv to remove
  */
@@ -513,6 +509,7 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 {
 	struct ConfItem *aconf = NULL;
 	rb_dlink_node *ptr;
+	time_t now;
 
 	if(IsChannelName(name))
 	{
@@ -540,8 +537,9 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 					       "%s has removed the global RESV for: [%s]",
 					       get_oper_name(source_p), name);
 			ilog(L_KLINE, "UR %s %s", get_oper_name(source_p), name);
-			if(aconf->created < rb_current_time())
-				aconf->created = rb_current_time();
+			now = rb_current_time();
+			if(aconf->created < now)
+				aconf->created = now;
 			else
 				aconf->created++;
 			aconf->hold = aconf->created;
@@ -554,7 +552,7 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 					(unsigned long)aconf->created,
 					0,
 					(int)(aconf->lifetime - aconf->created));
-			deactivate_conf(aconf, ptr);
+			deactivate_conf(aconf, ptr, now);
 			return;
 		}
 		else if(propagated && rb_dlink_list_length(&cluster_conf_list) > 0)
@@ -613,8 +611,9 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 					       "%s has removed the global RESV for: [%s]",
 					       get_oper_name(source_p), name);
 			ilog(L_KLINE, "UR %s %s", get_oper_name(source_p), name);
-			if(aconf->created < rb_current_time())
-				aconf->created = rb_current_time();
+			now = rb_current_time();
+			if(aconf->created < now)
+				aconf->created = now;
 			else
 				aconf->created++;
 			aconf->hold = aconf->created;
@@ -627,7 +626,7 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 					(unsigned long)aconf->created,
 					0,
 					(int)(aconf->lifetime - aconf->created));
-			deactivate_conf(aconf, ptr);
+			deactivate_conf(aconf, ptr, now);
 			return;
 		}
 		else if(propagated && rb_dlink_list_length(&cluster_conf_list) > 0)
