@@ -41,6 +41,7 @@
 #include "packet.h"
 #include "s_newconf.h"
 #include "ratelimit.h"
+#include "supported.h"
 
 #define FIELD_CHANNEL    0x0001
 #define FIELD_HOP        0x0002
@@ -69,8 +70,21 @@ struct Message who_msgtab = {
 	{mg_unreg, {m_who, 2}, mg_ignore, mg_ignore, mg_ignore, {m_who, 2}}
 };
 
+static int
+_modinit(void)
+{
+	add_isupport("WHOX", isupport_string, "");
+	return 0;
+}
+
+static void
+_moddeinit(void)
+{
+	delete_isupport("WHOX");
+}
+
 mapi_clist_av1 who_clist[] = { &who_msgtab, NULL };
-DECLARE_MODULE_AV1(who, NULL, NULL, who_clist, NULL, NULL, "$Revision: 3350 $");
+DECLARE_MODULE_AV1(who, _modinit, _moddeinit, who_clist, NULL, NULL, "$Revision: 3350 $");
 
 static void do_who_on_channel(struct Client *source_p, struct Channel *chptr,
 			      int server_oper, int member,
@@ -234,7 +248,7 @@ m_who(struct Client *client_p, struct Client *source_p, int parc, const char *pa
 		else
 			do_who(source_p, target_p, NULL, &fmt);
 
-		sendto_one(source_p, form_str(RPL_ENDOFWHO), 
+		sendto_one(source_p, form_str(RPL_ENDOFWHO),
 			   me.name, source_p->name, mask);
 		return 0;
 	}
@@ -486,7 +500,7 @@ do_who(struct Client *source_p, struct Client *target_p, struct membership *mspt
 			   source_p->name, msptr ? msptr->chptr->chname : "*",
 			   target_p->username, target_p->host,
 			   target_p->servptr->name, target_p->name, status,
-			   ConfigServerHide.flatten_links && !IsOper(source_p) && !IsExemptShide(source_p) ? 0 : target_p->hopcount, 
+			   ConfigServerHide.flatten_links && !IsOper(source_p) && !IsExemptShide(source_p) ? 0 : target_p->hopcount,
 			   target_p->info);
 	else
 	{
