@@ -45,6 +45,7 @@
 #include "s_newconf.h"
 #include "ipv4_from_ipv6.h"
 #include "ratelimit.h"
+#include "s_assert.h"
 
 static void do_whois(struct Client *client_p, struct Client *source_p, int parc, const char *parv[]);
 static void single_whois(struct Client *source_p, struct Client *target_p, int operspy);
@@ -59,11 +60,13 @@ struct Message whois_msgtab = {
 
 int doing_whois_hook;
 int doing_whois_global_hook;
+int doing_whois_channel_visibility_hook;
 
 mapi_clist_av1 whois_clist[] = { &whois_msgtab, NULL };
 mapi_hlist_av1 whois_hlist[] = {
-	{ "doing_whois",	&doing_whois_hook },
-	{ "doing_whois_global",	&doing_whois_global_hook },
+	{ "doing_whois",			&doing_whois_hook },
+	{ "doing_whois_global",			&doing_whois_global_hook },
+	{ "doing_whois_channel_visibility",	&doing_whois_channel_visibility_hook },
 	{ NULL, NULL }
 };
 
@@ -94,7 +97,7 @@ m_whois(struct Client *client_p, struct Client *source_p, int parc, const char *
 			{
 				sendto_one(source_p, form_str(RPL_LOAD2HI),
 					   me.name, source_p->name, "WHOIS");
-				sendto_one_numeric(source_p, RPL_ENDOFWHOIS, 
+				sendto_one_numeric(source_p, RPL_ENDOFWHOIS,
 						   form_str(RPL_ENDOFWHOIS), parv[2]);
 				return 0;
 			}
@@ -139,18 +142,18 @@ ms_whois(struct Client *client_p, struct Client *source_p, int parc, const char
 	if((target_p = find_client(parv[1])) == NULL)
 	{
 		sendto_one_numeric(source_p, ERR_NOSUCHSERVER,
-				   form_str(ERR_NOSUCHSERVER), 
+				   form_str(ERR_NOSUCHSERVER),
 				   IsDigit(parv[1][0]) ? "*" : parv[1]);
 		return 0;
 	}
 
 	/* if parv[1] isnt my client, or me, someone else is supposed
-	 * to be handling the request.. so send it to them 
+	 * to be handling the request.. so send it to them
 	 */
 	if(!MyClient(target_p) && !IsMe(target_p))
 	{
-		sendto_one(target_p, ":%s WHOIS %s :%s", 
-			   get_id(source_p, target_p), 
+		sendto_one(target_p, ":%s WHOIS %s :%s",
+			   get_id(source_p, target_p),
 			   get_id(target_p, target_p), parv[2]);
 		return 0;
 	}
@@ -167,8 +170,8 @@ ms_whois(struct Client *client_p, struct Client *source_p, int parc, const char
 
 /* do_whois
  *
- * inputs	- pointer to 
- * output	- 
+ * inputs	- pointer to
+ * output	-
  * side effects -
  */
 static void
@@ -189,11 +192,7 @@ do_whois(struct Client *client_p, struct Client *source_p, int parc, const char
 		nick++;
 	}
 
-	if(MyClient(source_p))
-		target_p = find_named_person(nick);
-	else
-		target_p = find_person(nick);
-
+	target_p = find_named_person(nick);
 	if(target_p != NULL)
 	{
 		if(operspy)
@@ -210,10 +209,10 @@ do_whois(struct Client *client_p, struct Client *source_p, int parc, const char
 	}
 	else
 		sendto_one_numeric(source_p, ERR_NOSUCHNICK,
-				   form_str(ERR_NOSUCHNICK), 
+				   form_str(ERR_NOSUCHNICK),
 				   nick);
 
-	sendto_one_numeric(source_p, RPL_ENDOFWHOIS, 
+	sendto_one_numeric(source_p, RPL_ENDOFWHOIS,
 			   form_str(RPL_ENDOFWHOIS), parv[1]);
 	return;
 }
@@ -239,7 +238,6 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 	char *t;
 	int tlen;
 	hook_data_client hdata;
-	int visible;
 	int extra_space = 0;
 #ifdef RB_IPV6
 	struct sockaddr_in ip4;
@@ -252,11 +250,11 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 	}
 
 	sendto_one_numeric(source_p, RPL_WHOISUSER, form_str(RPL_WHOISUSER),
-			   target_p->name, target_p->username, 
+			   target_p->name, target_p->username,
 			   target_p->host, target_p->info);
 
-	cur_len = mlen = rb_sprintf(buf, form_str(RPL_WHOISCHANNELS), 
-				    get_id(&me, source_p), get_id(source_p, source_p), 
+	cur_len = mlen = rb_sprintf(buf, form_str(RPL_WHOISCHANNELS),
+				    get_id(&me, source_p), get_id(source_p, source_p),
 				    target_p->name);
 
 	/* Make sure it won't overflow when sending it to the client
@@ -274,6 +272,9 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 
 	t = buf + mlen;
 
+	hdata.client = source_p;
+	hdata.target = target_p;
+
 	if (!IsService(target_p))
 	{
 		RB_DLINK_FOREACH(ptr, target_p->user->channel.head)
@@ -281,9 +282,12 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 			msptr = ptr->data;
 			chptr = msptr->chptr;
 
-			visible = ShowChannel(source_p, chptr);
+			hdata.chptr = chptr;
 
-			if(visible || operspy)
+			hdata.approved = ShowChannel(source_p, chptr);
+			call_hook(doing_whois_channel_visibility_hook, &hdata);
+
+			if(hdata.approved || operspy)
 			{
 				if((cur_len + strlen(chptr->chname) + 3) > (BUFSIZE - 5))
 				{
@@ -293,7 +297,7 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 				}
 
 				tlen = rb_sprintf(t, "%s%s%s ",
-						visible ? "" : "!",
+						hdata.approved ? "" : "!",
 						find_channel_status(msptr, 1),
 						chptr->chname);
 				t += tlen;
@@ -322,14 +326,30 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 				    GlobalSetOptions.operstring));
 	}
 
+	if(MyClient(target_p) && !EmptyString(target_p->localClient->opername) && IsOper(target_p) && IsOper(source_p))
+	{
+		char buf[512];
+		rb_snprintf(buf, sizeof(buf), "is opered as %s, privset %s",
+			    target_p->localClient->opername, target_p->localClient->privset->name);
+		sendto_one_numeric(source_p, RPL_WHOISSPECIAL, form_str(RPL_WHOISSPECIAL),
+				   target_p->name, buf);
+	}
+
 	if(IsSSLClient(target_p))
+	{
+		char cbuf[256] = "is using a secure connection";
+
+		if (MyClient(target_p) && target_p->localClient->cipher_string != NULL)
+			rb_snprintf_append(cbuf, sizeof(cbuf), " [%s]", target_p->localClient->cipher_string);
+
 		sendto_one_numeric(source_p, RPL_WHOISSECURE, form_str(RPL_WHOISSECURE),
-				   target_p->name);
-	if((source_p == target_p || IsOper(source_p)) &&
-			target_p->certfp != NULL)
-		sendto_one_numeric(source_p, RPL_WHOISCERTFP,
-				form_str(RPL_WHOISCERTFP),
-				target_p->name, target_p->certfp);
+				   target_p->name, cbuf);
+		if((source_p == target_p || IsOper(source_p)) &&
+				target_p->certfp != NULL)
+			sendto_one_numeric(source_p, RPL_WHOISCERTFP,
+					form_str(RPL_WHOISCERTFP),
+					target_p->name, target_p->certfp);
+	}
 
 	if(MyClient(target_p))
 	{
@@ -365,9 +385,9 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 #endif /* RB_IPV6 */
 
 		sendto_one_numeric(source_p, RPL_WHOISIDLE, form_str(RPL_WHOISIDLE),
-				   target_p->name, 
-				   rb_current_time() - target_p->localClient->last, 
-				   target_p->localClient->firsttime);
+				   target_p->name,
+				   (long)(rb_current_time() - target_p->localClient->last),
+				   (unsigned long)target_p->localClient->firsttime);
 	}
 	else
 	{
@@ -386,13 +406,10 @@ single_whois(struct Client *source_p, struct Client *target_p, int operspy)
 			sendto_one_numeric(source_p, RPL_WHOISACTUALLY,
 					   form_str(RPL_WHOISACTUALLY),
 					   target_p->name, target_p->sockhost);
-			
+
 		}
 	}
 
-	hdata.client = source_p;
-	hdata.target = target_p;
-
 	/* doing_whois_hook must only be called for local clients,
 	 * doing_whois_global_hook must only be called for local targets
 	 */
