@@ -122,6 +122,9 @@ void
 bandb_add(bandb_type type, struct Client *source_p, const char *mask1,
 	  const char *mask2, const char *reason, const char *oper_reason, int perm)
 {
+	if(bandb_helper == NULL)
+		return;
+
 	static char buf[BUFSIZE];
 
 	rb_snprintf(buf, sizeof(buf), "%c %s ", bandb_add_letter[type], mask1);
@@ -145,6 +148,9 @@ static char bandb_del_letter[LAST_BANDB_TYPE] = {
 void
 bandb_del(bandb_type type, const char *mask1, const char *mask2)
 {
+	if(bandb_helper == NULL)
+		return;
+
 	static char buf[BUFSIZE];
 
 	buf[0] = '\0';
@@ -215,7 +221,7 @@ bandb_check_kline(struct ConfItem *aconf)
 	int aftype;
 	const char *p;
 
-	aftype = parse_netmask(aconf->host, (struct sockaddr *)&daddr, NULL);
+	aftype = parse_netmask(aconf->host, &daddr, NULL);
 
 	if(aftype != HM_HOST)
 	{
@@ -254,10 +260,9 @@ static int
 bandb_check_dline(struct ConfItem *aconf)
 {
 	struct rb_sockaddr_storage daddr;
-/* 	struct ConfItem *dconf; */
 	int bits;
 
-	if(!parse_netmask(aconf->host, (struct sockaddr *)&daddr, &bits))
+	if(!parse_netmask(aconf->host, &daddr, &bits))
 		return 0;
 
 	return 1;
@@ -384,7 +389,7 @@ bandb_handle_failure(rb_helper *helper, char **parv, int parc)
 {
 	if(server_state_foreground)
 		fprintf(stderr, "bandb - bandb failure: %s\n", parv[1]);
-	
+
 	ilog(L_MAIN, "bandb - bandb failure: %s", parv[1]);
 	sendto_realops_snomask(SNO_GENERAL, L_ALL, "bandb - bandb failure: %s", parv[1]);
 	exit(1);
@@ -418,6 +423,7 @@ bandb_parse(rb_helper *helper)
 
 		case 'C':
 			bandb_handle_clear();
+			break;
 		case 'F':
 			bandb_handle_finish();
 			break;
