@@ -55,6 +55,7 @@
 #include "scache.h"
 #include "irc_dictionary.h"
 #include "sslproc.h"
+#include "s_assert.h"
 
 #define DEBUG_EXITED_CLIENTS
 
@@ -78,6 +79,7 @@ static rb_bh *pclient_heap = NULL;
 static rb_bh *user_heap = NULL;
 static rb_bh *away_heap = NULL;
 static char current_uid[IDLEN];
+static int32_t current_connid = 0;
 
 struct Dictionary *nd_dict = NULL;
 
@@ -161,7 +163,18 @@ make_client(struct Client *from)
 
 		client_p->localClient->F = NULL;
 
-		client_p->preClient = rb_bh_alloc(pclient_heap);;
+		if(current_connid+1 == 0)
+			current_connid++;
+
+		client_p->localClient->connid = ++current_connid;
+
+		if(current_connid+1 == 0)
+			current_connid++;
+
+		client_p->localClient->zconnid = ++current_connid;
+		add_to_cli_connid_hash(client_p);
+
+		client_p->preClient = rb_bh_alloc(pclient_heap);
 
 		/* as good a place as any... */
 		rb_dlinkAdd(client_p, &client_p->localClient->tnode, &unknown_list);
@@ -172,7 +185,7 @@ make_client(struct Client *from)
 		client_p->preClient = NULL;
 		client_p->from = from;	/* 'from' of local client is self! */
 	}
-	
+
 	SetUnknown(client_p);
 	strcpy(client_p->username, "unknown");
 
@@ -192,7 +205,8 @@ free_pre_client(struct Client *client_p)
 	blptr = client_p->preClient->dnsbl_listed;
 	if (blptr != NULL)
 		unref_blacklist(blptr);
-	abort_blacklist_queries(client_p);
+	s_assert(rb_dlink_list_length(&client_p->preClient->dnsbl_queries) == 0);
+
 	rb_bh_free(pclient_heap, client_p->preClient);
 	client_p->preClient = NULL;
 }
@@ -218,9 +232,9 @@ free_local_client(struct Client *client_p)
 		client_p->localClient->listener = 0;
 	}
 
+	del_from_cli_connid_hash(client_p);
 	if(client_p->localClient->F != NULL)
 	{
-		del_from_cli_fd_hash(client_p);
 		rb_close(client_p->localClient->F);
 	}
 
@@ -241,15 +255,19 @@ free_local_client(struct Client *client_p)
 
 	if(IsSSL(client_p))
 	    ssld_decrement_clicount(client_p->localClient->ssl_ctl);
-	    
+
+	rb_free(client_p->localClient->cipher_string);
+
 	if(IsCapable(client_p, CAP_ZIP))
 	    ssld_decrement_clicount(client_p->localClient->z_ctl);
 
+	rb_free(client_p->localClient->zipstats);
+
 	rb_bh_free(lclient_heap, client_p->localClient);
 	client_p->localClient = NULL;
 }
 
-void
+static void
 free_client(struct Client *client_p)
 {
 	s_assert(NULL != client_p);
@@ -266,7 +284,7 @@ free_client(struct Client *client_p)
  *
  * inputs       - NOT USED (from event)
  * output       - next time_t when check_pings() should be called again
- * side effects - 
+ * side effects -
  *
  *
  * A PING can be sent to clients as necessary.
@@ -299,7 +317,7 @@ check_pings(void *notused)
  *
  * inputs	- pointer to list to check
  * output	- NONE
- * side effects	- 
+ * side effects	-
  */
 static void
 check_pings_list(rb_dlink_list * list)
@@ -433,8 +451,8 @@ notify_banned_client(struct Client *client_p, struct ConfItem *aconf, int ban)
 	else
 		sendto_one(client_p, form_str(ERR_YOUREBANNEDCREEP),
 			   me.name, client_p->name, reason);
-	
-	exit_client(client_p, client_p, &me, 
+
+	exit_client(client_p, client_p, &me,
 			EmptyString(ConfigFileEntry.kline_reason) ? exit_reason :
 			 ConfigFileEntry.kline_reason);
 }
@@ -664,7 +682,7 @@ resv_nick_fnc(const char *mask, const char *reason, int temp_time)
 
 			invalidate_bancache_user(client_p);
 
-			sendto_common_channels_local(client_p, NOCAPS, ":%s!%s@%s NICK :%s",
+			sendto_common_channels_local(client_p, NOCAPS, NOCAPS, ":%s!%s@%s NICK :%s",
 				client_p->name, client_p->username, client_p->host, nick);
 			sendto_server(client_p, NULL, CAP_TS6, NOCAPS, ":%s NICK %s :%ld",
 				use_id(client_p), nick, (long) client_p->tsinfo);
@@ -693,7 +711,7 @@ resv_nick_fnc(const char *mask, const char *reason, int temp_time)
  *
  * input	- pointer to client
  * output	- NONE
- * side effects	- 
+ * side effects	-
  */
 static void
 update_client_exit_stats(struct Client *client_p)
@@ -724,7 +742,7 @@ update_client_exit_stats(struct Client *client_p)
  *
  * input	- pointer to client to release
  * output	- NONE
- * side effects	- 
+ * side effects	-
  */
 static void
 release_client_state(struct Client *client_p)
@@ -773,6 +791,38 @@ remove_client_from_list(struct Client *client_p)
 }
 
 
+/* clean_nick()
+ *
+ * input	- nickname to check, flag for nick from local client
+ * output	- 0 if erroneous, else 1
+ * side effects -
+ */
+int
+clean_nick(const char *nick, int loc_client)
+{
+	int len = 0;
+
+	/* nicks cant start with a digit or -, and must have a length */
+	if(*nick == '-' || *nick == '\0')
+		return 0;
+
+	if(loc_client && IsDigit(*nick))
+		return 0;
+
+	for(; *nick; nick++)
+	{
+		len++;
+		if(!IsNickChar(*nick))
+			return 0;
+	}
+
+	/* nicklen is +1 */
+	if(len >= NICKLEN && (unsigned int)len >= ConfigFileEntry.nicklen)
+		return 0;
+
+	return 1;
+}
+
 /*
  * find_person	- find person by (nick)name.
  * inputs	- pointer to name
@@ -805,8 +855,8 @@ find_named_person(const char *name)
 
 
 /*
- * find_chasing - find the client structure for a nick name (user) 
- *      using history mechanism if necessary. If the client is not found, 
+ * find_chasing - find the client structure for a nick name (user)
+ *      using history mechanism if necessary. If the client is not found,
  *      an error message (NO SUCH NICK) is generated. If the client was found
  *      through the history, chasing will be 1 and otherwise 0.
  */
@@ -870,7 +920,7 @@ get_client_name(struct Client *client, int showip)
 		if(!irccmp(client->name, client->host))
 			return client->name;
 
-		if(ConfigFileEntry.hide_spoof_ips && 
+		if(ConfigFileEntry.hide_spoof_ips &&
 		   showip == SHOW_IP && IsIPSpoof(client))
 			showip = MASK_IP;
 		if(IsAnyServer(client))
@@ -880,8 +930,8 @@ get_client_name(struct Client *client, int showip)
 		switch (showip)
 		{
 		case SHOW_IP:
-			rb_snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]", 
-				   client->name, client->username, 
+			rb_snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]",
+				   client->name, client->username,
 				   client->sockhost);
 			break;
 		case MASK_IP:
@@ -895,12 +945,12 @@ get_client_name(struct Client *client, int showip)
 		return nbuf;
 	}
 
-	/* As pointed out by Adel Mezibra 
+	/* As pointed out by Adel Mezibra
 	 * Neph|l|m@EFnet. Was missing a return here.
 	 */
 	return client->name;
 }
-	
+
 /* log_client_name()
  *
  * This version is the same as get_client_name, but doesnt contain the
@@ -926,10 +976,6 @@ log_client_name(struct Client *target_p, int showip)
 				   target_p->username, target_p->sockhost);
 			break;
 
-		case MASK_IP:
-			rb_snprintf(nbuf, sizeof(nbuf), "%s[%s@255.255.255.255]",
-				   target_p->name, target_p->username);
-
 		default:
 			rb_snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]", target_p->name,
 				   target_p->username, target_p->host);
@@ -976,7 +1022,7 @@ free_exited_clients(void *unused)
 				if(abt->client == target_p)
 				{
 					s_assert(0);
-					sendto_realops_snomask(SNO_GENERAL, L_ALL, 
+					sendto_realops_snomask(SNO_GENERAL, L_ALL,
 						"On abort_list: %s stat: %u flags: %u/%u handler: %c",
 						target_p->name, (unsigned int) target_p->status,
 						target_p->flags, target_p->flags2, target_p->handler);
@@ -1023,53 +1069,13 @@ free_exited_clients(void *unused)
 		rb_dlinkDestroy(ptr, &dead_remote_list);
 	}
 #endif
-	
+
 }
 
 /*
-** Recursively send QUITs and SQUITs for source_p and all its dependent clients
-** and servers to those servers that need them.  A server needs the client
-** QUITs if it can't figure them out from the SQUIT (ie pre-TS4) or if it
-** isn't getting the SQUIT because of @#(*&@)# hostmasking.  With TS4, once
-** a link gets a SQUIT, it doesn't need any QUIT/SQUITs for clients depending
-** on that one -orabidoo
-*/
-static void
-recurse_send_quits(struct Client *client_p, struct Client *source_p, 
-		   struct Client *to, const char *comment1,
-		   const char *comment)
-{
-	struct Client *target_p;
-	rb_dlink_node *ptr, *ptr_next;
-	/* If this server can handle quit storm (QS) removal
-	 * of dependents, just send the SQUIT
-	 */
-
-	if(IsCapable(to, CAP_QS))
-	{
-		sendto_one(to, "SQUIT %s :%s",
-			   get_id(source_p, to), comment);
-	}
-	else
-	{
-		RB_DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->serv->users.head)
-		{
-			target_p = ptr->data;
-			sendto_one(to, ":%s QUIT :%s", target_p->name, comment1);
-		}
-		RB_DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->serv->servers.head)
-		{
-			target_p = ptr->data;
-			recurse_send_quits(client_p, target_p, to, comment1, comment);
-		}
-		sendto_one(to, "SQUIT %s :%s", source_p->name, comment);
-	}
-}
-
-/* 
 ** Remove all clients that depend on source_p; assumes all (S)QUITs have
-** already been sent.  we make sure to exit a server's dependent clients 
-** and servers before the server itself; exit_one_client takes care of 
+** already been sent.  we make sure to exit a server's dependent clients
+** and servers before the server itself; exit_one_client takes care of
 ** actually removing things off llists.   tweaked from +CSr31  -orabidoo
 */
 /*
@@ -1110,7 +1116,7 @@ recurse_remove_clients(struct Client *source_p, const char *comment)
 			if(!IsDead(target_p) && !IsClosing(target_p))
 				exit_remote_client(NULL, target_p, &me, comment);
 		}
-	}	
+	}
 
 	RB_DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->serv->servers.head)
 	{
@@ -1122,7 +1128,7 @@ recurse_remove_clients(struct Client *source_p, const char *comment)
 
 /*
 ** Remove *everything* that depends on source_p, from all lists, and sending
-** all necessary QUITs and SQUITs.  source_p itself is still on the lists,
+** all necessary SQUITs.  source_p itself is still on the lists,
 ** and its SQUITs have been sent except for the upstream one  -orabidoo
 */
 static void
@@ -1137,11 +1143,10 @@ remove_dependents(struct Client *client_p,
 	{
 		to = ptr->data;
 
-		if(IsMe(to) || to == source_p->from || 
-		   (to == client_p && IsCapable(to, CAP_QS)))
+		if(IsMe(to) || to == source_p->from || to == client_p)
 			continue;
 
-		recurse_send_quits(client_p, source_p, to, comment1, comment);
+		sendto_one(to, "SQUIT %s :%s", get_id(source_p, to), comment);
 	}
 
 	recurse_remove_clients(source_p, comment1);
@@ -1161,7 +1166,7 @@ exit_aborted_clients(void *unused)
 			if(rb_dlinkFind(abt->client, &dead_list))
 			{
 				s_assert(0);
-				sendto_realops_snomask(SNO_GENERAL, L_ALL, 
+				sendto_realops_snomask(SNO_GENERAL, L_ALL,
 					"On dead_list: %s stat: %u flags: %u/%u handler: %c",
 					abt->client->name, (unsigned int) abt->client->status,
 					abt->client->flags, abt->client->flags2, abt->client->handler);
@@ -1228,7 +1233,7 @@ exit_generic_client(struct Client *client_p, struct Client *source_p, struct Cli
 	if(IsOper(source_p))
 		rb_dlinkFindDestroy(source_p, &oper_list);
 
-	sendto_common_channels_local(source_p, NOCAPS, ":%s!%s@%s QUIT :%s",
+	sendto_common_channels_local(source_p, NOCAPS, NOCAPS, ":%s!%s@%s QUIT :%s",
 				     source_p->name,
 				     source_p->username, source_p->host, comment);
 
@@ -1259,7 +1264,7 @@ exit_generic_client(struct Client *client_p, struct Client *source_p, struct Cli
 	remove_client_from_list(source_p);
 }
 
-/* 
+/*
  * Assumes IsPerson(source_p) && !MyConnect(source_p)
  */
 
@@ -1268,7 +1273,7 @@ exit_remote_client(struct Client *client_p, struct Client *source_p, struct Clie
 		   const char *comment)
 {
 	exit_generic_client(client_p, source_p, from, comment);
-	
+
 	if(source_p->servptr && source_p->servptr->serv)
 	{
 		rb_dlinkDelete(&source_p->lnode, &source_p->servptr->serv->users);
@@ -1298,6 +1303,7 @@ exit_unknown_client(struct Client *client_p, struct Client *source_p, struct Cli
 		  const char *comment)
 {
 	delete_auth_queries(source_p);
+	abort_blacklist_queries(source_p);
 	if (source_p->localClient->dnsquery)
 	{
 		delete_resolver_queries(source_p->localClient->dnsquery);
@@ -1326,13 +1332,13 @@ exit_unknown_client(struct Client *client_p, struct Client *source_p, struct Cli
 }
 
 static int
-exit_remote_server(struct Client *client_p, struct Client *source_p, struct Client *from, 
+exit_remote_server(struct Client *client_p, struct Client *source_p, struct Client *from,
 		  const char *comment)
 {
 	static char comment1[(HOSTLEN*2)+2];
 	static char newcomment[BUFSIZE];
 	struct Client *target_p;
-	
+
 	if(ConfigServerHide.flatten_links)
 		strcpy(comment1, "*.net *.split");
 	else
@@ -1355,11 +1361,11 @@ exit_remote_server(struct Client *client_p, struct Client *source_p, struct Clie
 
 	rb_dlinkFindDestroy(source_p, &global_serv_list);
 	target_p = source_p->from;
-	
+
 	if(target_p != NULL && IsServer(target_p) && target_p != client_p &&
 	   !IsMe(target_p) && (source_p->flags & FLAGS_KILLED) == 0)
 	{
-		sendto_one(target_p, ":%s SQUIT %s :%s", 
+		sendto_one(target_p, ":%s SQUIT %s :%s",
 			   get_id(from, target_p), get_id(source_p, target_p),
 			   comment);
 	}
@@ -1368,9 +1374,9 @@ exit_remote_server(struct Client *client_p, struct Client *source_p, struct Clie
 		del_from_id_hash(source_p->id, source_p);
 
 	del_from_client_hash(source_p->name, source_p);
-	remove_client_from_list(source_p);  
+	remove_client_from_list(source_p);
 	scache_split(source_p->serv->nameinfo);
-	
+
 	SetDead(source_p);
 #ifdef DEBUG_EXITED_CLIENTS
 	rb_dlinkAddAlloc(source_p, &dead_remote_list);
@@ -1381,7 +1387,7 @@ exit_remote_server(struct Client *client_p, struct Client *source_p, struct Clie
 }
 
 static int
-qs_server(struct Client *client_p, struct Client *source_p, struct Client *from, 
+qs_server(struct Client *client_p, struct Client *source_p, struct Client *from,
 		  const char *comment)
 {
 	if(source_p->servptr && source_p->servptr->serv)
@@ -1390,30 +1396,30 @@ qs_server(struct Client *client_p, struct Client *source_p, struct Client *from,
 		s_assert(0);
 
 	rb_dlinkFindDestroy(source_p, &global_serv_list);
-	
+
 	if(has_id(source_p))
 		del_from_id_hash(source_p->id, source_p);
 
 	del_from_client_hash(source_p->name, source_p);
-	remove_client_from_list(source_p);  
+	remove_client_from_list(source_p);
 	scache_split(source_p->serv->nameinfo);
-	
+
 	SetDead(source_p);
-	rb_dlinkAddAlloc(source_p, &dead_list);	
+	rb_dlinkAddAlloc(source_p, &dead_list);
 	return 0;
 }
 
 static int
-exit_local_server(struct Client *client_p, struct Client *source_p, struct Client *from, 
+exit_local_server(struct Client *client_p, struct Client *source_p, struct Client *from,
 		  const char *comment)
 {
 	static char comment1[(HOSTLEN*2)+2];
 	static char newcomment[BUFSIZE];
 	unsigned int sendk, recvk;
-	
+
 	rb_dlinkDelete(&source_p->localClient->tnode, &serv_list);
 	rb_dlinkFindDestroy(source_p, &global_serv_list);
-	
+
 	sendk = source_p->localClient->sendK;
 	recvk = source_p->localClient->receiveK;
 
@@ -1430,7 +1436,7 @@ exit_local_server(struct Client *client_p, struct Client *source_p, struct Clien
 		sendto_one(source_p, "ERROR :Closing Link: 127.0.0.1 %s (%s)",
 			   source_p->name, comment);
 	}
-	
+
 	if(source_p->servptr && source_p->servptr->serv)
 		rb_dlinkDelete(&source_p->lnode, &source_p->servptr->serv->servers);
 	else
@@ -1438,7 +1444,7 @@ exit_local_server(struct Client *client_p, struct Client *source_p, struct Clien
 
 
 	close_connection(source_p);
-	
+
 	if(ConfigServerHide.flatten_links)
 		strcpy(comment1, "*.net *.split");
 	else
@@ -1457,21 +1463,21 @@ exit_local_server(struct Client *client_p, struct Client *source_p, struct Clien
 
 	ilog(L_SERVER, "%s was connected for %ld seconds.  %d/%d sendK/recvK.",
 	     source_p->name, (long) rb_current_time() - source_p->localClient->firsttime, sendk, recvk);
-        
+
 	if(has_id(source_p))
 		del_from_id_hash(source_p->id, source_p);
 
 	del_from_client_hash(source_p->name, source_p);
 	remove_client_from_list(source_p);
 	scache_split(source_p->serv->nameinfo);
-	
+
 	SetDead(source_p);
 	rb_dlinkAddAlloc(source_p, &dead_list);
 	return 0;
 }
 
 
-/* 
+/*
  * This assumes IsPerson(source_p) == TRUE && MyConnect(source_p) == TRUE
  */
 
@@ -1587,8 +1593,8 @@ exit_client(struct Client *client_p,	/* The local client originating the
 		/* IsUnknown || IsConnecting || IsHandShake */
 		else if(!IsReject(source_p))
 			return exit_unknown_client(client_p, source_p, from, comment);
-	} 
-	else 
+	}
+	else
 	{
 		/* Remotes */
 		if(IsPerson(source_p))
@@ -1629,9 +1635,9 @@ count_remote_client_memory(size_t * count, size_t * remote_client_memory_used)
 
 /*
  * accept processing, this adds a form of "caller ID" to ircd
- * 
+ *
  * If a client puts themselves into "caller ID only" mode,
- * only clients that match a client pointer they have put on 
+ * only clients that match a client pointer they have put on
  * the accept list will be allowed to message them.
  *
  * [ source.on_allow_list ] -> [ target1 ] -> [ target2 ]
@@ -1642,7 +1648,7 @@ count_remote_client_memory(size_t * count, size_t * remote_client_memory_used)
  * each source client then has a back pointer pointing back
  * to the client that has it on its accept list.
  * This allows for exit_one_client to remove these now bogus entries
- * from any client having an accept on them. 
+ * from any client having an accept on them.
  */
 /*
  * del_all_accepts
@@ -1683,7 +1689,7 @@ del_all_accepts(struct Client *client_p)
 
 /*
  * show_ip() - asks if the true IP should be shown when source is
- *             asking for info about target 
+ *             asking for info about target
  *
  * Inputs	- source_p who is asking
  *		- target_p who do we want the info on
@@ -1728,6 +1734,18 @@ show_ip_conf(struct ConfItem *aconf, struct Client *source_p)
 		return 1;
 }
 
+int
+show_ip_whowas(struct Whowas *whowas, struct Client *source_p)
+{
+	if(whowas->flags & WHOWAS_IP_SPOOFING)
+		if(ConfigFileEntry.hide_spoof_ips || !MyOper(source_p))
+			return 0;
+	if(whowas->flags & WHOWAS_DYNSPOOF)
+		if(!IsOper(source_p))
+			return 0;
+	return 1;
+}
+
 /*
  * make_user
  *
@@ -1774,7 +1792,7 @@ make_server(struct Client *client_p)
 
 /*
  * free_user
- * 
+ *
  * inputs	- pointer to user struct
  *		- pointer to client struct
  * output	- none
@@ -1804,7 +1822,7 @@ free_user(struct User *user, struct Client *client_p)
 					     client_p->host,
 					     (unsigned long) user,
 					     (unsigned long) user->invited.head,
-					     (unsigned long) user->channel.head, 
+					     (unsigned long) user->channel.head,
 					     rb_dlink_list_length(&user->channel),
 					     user->refcnt);
 			s_assert(!user->refcnt);
@@ -1820,7 +1838,7 @@ void
 allocate_away(struct Client *client_p)
 {
 	if(client_p->user->away == NULL)
-		client_p->user->away = rb_bh_alloc(away_heap);	
+		client_p->user->away = rb_bh_alloc(away_heap);
 }
 
 
@@ -1896,7 +1914,7 @@ close_connection(struct Client *client_p)
 	s_assert(MyConnect(client_p));
 	if(!MyConnect(client_p))
 		return;
-	
+
 	if(IsServer(client_p))
 	{
 		struct server_conf *server_p;
@@ -1941,7 +1959,7 @@ close_connection(struct Client *client_p)
 		if(!IsIOError(client_p))
 			send_queued(client_p);
 
-		del_from_cli_fd_hash(client_p);
+		del_from_cli_connid_hash(client_p);
 		rb_close(client_p->localClient->F);
 		client_p->localClient->F = NULL;
 	}
