@@ -5,7 +5,7 @@
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
  *  Copyright (C) 2002-2008 ircd-ratbox development team
- *  Copyright (C) 2005-2008 charybdis development team
+ *  Copyright (C) 2005-2013 charybdis development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -106,8 +106,8 @@ int doremotd = 0;
 int kline_queued = 0;
 int server_state_foreground = 0;
 int opers_see_all_users = 0;
-int ssl_ok = 0;
-int zlib_ok = 1;
+int ircd_ssl_ok = 0;
+int ircd_zlib_ok = 1;
 
 int testing_conf = 0;
 time_t startup_time;
@@ -149,28 +149,6 @@ ircd_shutdown(const char *reason)
 	exit(0);
 }
 
-/*
- * print_startup - print startup information
- */
-static void
-print_startup(int pid)
-{
-	inotice("now running in %s mode from %s as pid %d ...",
-	       !server_state_foreground ? "background" : "foreground",
-        	ConfigFileEntry.dpath, pid);
-
-	/* let the parent process know the initialization was successful
-	 * -- jilles */
-	if (!server_state_foreground)
-		write(0, ".", 1);
-	fclose(stdin);
-	fclose(stdout);
-	fclose(stderr);
-	open("/dev/null", O_RDWR);
-	dup2(0, 1);
-	dup2(0, 2);
-}
-
 /*
  * init_sys
  *
@@ -202,17 +180,18 @@ init_sys(void)
 static int
 make_daemon(void)
 {
-	int pid;
-	int pip[2];
-	char c;
+	int pid, nullfd, fdx;
 
-	if (pipe(pip) < 0)
+	/* The below is approximately what daemon(1, 0) does, but
+	   we need control over the parent after forking to print
+	   the startup message -- Aaron */
+
+	if((nullfd = open("/dev/null", O_RDWR)) < 0)
 	{
-		perror("pipe");
+		perror("open /dev/null");
 		exit(EXIT_FAILURE);
 	}
-	dup2(pip[1], 0);
-	close(pip[1]);
+
 	if((pid = fork()) < 0)
 	{
 		perror("fork");
@@ -220,22 +199,20 @@ make_daemon(void)
 	}
 	else if(pid > 0)
 	{
-		close(0);
-		/* Wait for initialization to finish, successfully or
-		 * unsuccessfully. Until this point the child may still
-		 * write to stdout/stderr.
-		 * -- jilles */
-		if (read(pip[0], &c, 1) > 0)
-			exit(EXIT_SUCCESS);
-		else
-			exit(EXIT_FAILURE);
+		inotice("now running in background mode from %s as pid %d ...",
+		        ConfigFileEntry.dpath, pid);
+
+		exit(EXIT_SUCCESS);
 	}
 
-	close(pip[0]);
-	setsid();
-/*	fclose(stdin);
-	fclose(stdout);
-	fclose(stderr); */
+	for(fdx = 0; fdx <= 2; fdx++)
+		if (fdx != nullfd)
+			(void) dup2(nullfd, fdx);
+
+	if(nullfd > 2)
+		(void) close(nullfd);
+
+	(void) setsid();
 
 	return 0;
 }
@@ -291,7 +268,7 @@ check_rehash(void *unused)
  *
  * inputs       - none
  * output       - none
- * side effects - This sets all global set options needed 
+ * side effects - This sets all global set options needed
  */
 static void
 initialize_global_set_options(void)
@@ -309,10 +286,7 @@ initialize_global_set_options(void)
 	GlobalSetOptions.spam_time = MIN_JOIN_LEAVE_TIME;
 	GlobalSetOptions.spam_num = MAX_JOIN_LEAVE_COUNT;
 
-	if(ConfigFileEntry.default_floodcount)
-		GlobalSetOptions.floodcount = ConfigFileEntry.default_floodcount;
-	else
-		GlobalSetOptions.floodcount = 10;
+	GlobalSetOptions.floodcount = ConfigFileEntry.default_floodcount;
 
 	split_servers = ConfigChannel.default_split_server_count;
 	split_users = ConfigChannel.default_split_user_count;
@@ -324,10 +298,7 @@ initialize_global_set_options(void)
 		splitchecking = 1;
 	}
 
-	if(ConfigFileEntry.default_ident_timeout)
-		GlobalSetOptions.ident_timeout = ConfigFileEntry.default_ident_timeout;
-	else
-		GlobalSetOptions.ident_timeout = IDENT_TIMEOUT;
+	GlobalSetOptions.ident_timeout = ConfigFileEntry.default_ident_timeout;
 
 	rb_strlcpy(GlobalSetOptions.operstring,
 		ConfigFileEntry.default_operstring,
@@ -494,6 +465,7 @@ seed_with_urandom(void)
 			srand(seed);
 			return 1;
 		}
+		close(fd);
 	}
 	return 0;
 }
@@ -501,7 +473,7 @@ seed_with_urandom(void)
 static void
 seed_with_clock(void)
 {
- 	const struct timeval *tv;	
+ 	const struct timeval *tv;
 	rb_set_time();
 	tv = rb_current_time_tv();
 	srand(tv->tv_sec ^ (tv->tv_usec | (getpid() << 20)));
@@ -614,21 +586,18 @@ main(int argc, char *argv[])
 	{
 		check_pidfile(pidFileName);
 
-		if(!server_state_foreground)
-			make_daemon();
 		inotice("starting %s ...", ircd_version);
 		inotice("%s", rb_lib_version());
+
+		if(!server_state_foreground)
+			make_daemon();
 	}
 
 	/* Init the event subsystem */
 	rb_lib_init(ircd_log_cb, ircd_restart_cb, ircd_die_cb, !server_state_foreground, maxconnections, DNODE_HEAP_SIZE, FD_HEAP_SIZE);
 	rb_linebuf_init(LINEBUF_HEAP_SIZE);
 
-	if(ConfigFileEntry.use_egd && (ConfigFileEntry.egdpool_path != NULL))
-	{
-		rb_init_prng(ConfigFileEntry.egdpool_path, RB_PRNG_EGD);
-	} else
-		rb_init_prng(NULL, RB_PRNG_DEFAULT);
+	rb_init_prng(NULL, RB_PRNG_DEFAULT);
 
 	seed_random(NULL);
 
@@ -667,8 +636,8 @@ main(int argc, char *argv[])
 	read_conf_files(YES);	/* cold start init conf files */
 #ifndef STATIC_MODULES
 
-	mod_add_path(MODULE_DIR); 
-	mod_add_path(MODULE_DIR "/autoload"); 
+	mod_add_path(MODULE_DIR);
+	mod_add_path(MODULE_DIR "/autoload");
 #endif
 
 	init_isupport();
@@ -707,13 +676,13 @@ main(int argc, char *argv[])
 	if(ServerInfo.ssl_cert != NULL && ServerInfo.ssl_private_key != NULL)
 	{
 		/* just do the rb_setup_ssl_server to validate the config */
-		if(!rb_setup_ssl_server(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params))
+		if(!rb_setup_ssl_server(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list))
 		{
 			ilog(L_MAIN, "WARNING: Unable to setup SSL.");
-			ssl_ok = 0;
+			ircd_ssl_ok = 0;
 		}
 		else
-			ssl_ok = 1;
+			ircd_ssl_ok = 1;
 	}
 
 	if (testing_conf)
@@ -756,7 +725,9 @@ main(int argc, char *argv[])
 	if(splitmode)
 		check_splitmode_ev = rb_event_add("check_splitmode", check_splitmode, NULL, 5);
 
-	print_startup(getpid());
+	if(server_state_foreground)
+		inotice("now running in foreground mode from %s as pid %d ...",
+		        ConfigFileEntry.dpath, getpid());
 
 	rb_lib_loop(0);
 
