@@ -52,11 +52,13 @@
 #include "bandbi.h"
 #include "operhash.h"
 #include "chmode.h"
+#include "hook.h"
+#include "s_assert.h"
 
 struct config_server_hide ConfigServerHide;
 
 extern int yyparse(void);		/* defined in y.tab.c */
-extern char linebuf[];
+extern char yy_linebuf[16384];		/* defined in ircd_lexer.l */
 
 #ifndef INADDR_NONE
 #define INADDR_NONE ((unsigned int) 0xffffffff)
@@ -73,7 +75,7 @@ rb_dlink_list service_list;
 /* internally defined functions */
 static void set_default_conf(void);
 static void validate_conf(void);
-static void read_conf(FILE *);
+static void read_conf(void);
 static void clear_out_old_conf(void);
 
 static void expire_prop_bans(void *list);
@@ -84,6 +86,7 @@ FILE *conf_fbfile_in;
 extern char yytext[];
 
 static int verify_access(struct Client *client_p, const char *username);
+static struct ConfItem *find_address_conf_by_client(struct Client *client_p, const char *username);
 static int attach_iline(struct Client *, struct ConfItem *);
 
 void
@@ -182,10 +185,10 @@ check_client(struct Client *client_p, struct Client *source_p, const char *usern
 
 	if((i = verify_access(source_p, username)))
 	{
-		ilog(L_FUSER, "Access denied: %s[%s]", 
+		ilog(L_FUSER, "Access denied: %s[%s]",
 		     source_p->name, source_p->sockhost);
 	}
-	
+
 	switch (i)
 	{
 	case SOCKET_ERROR:
@@ -205,7 +208,7 @@ check_client(struct Client *client_p, struct Client *source_p, const char *usern
 
 		ilog(L_FUSER, "Too many local connections from %s!%s%s@%s",
 			source_p->name, IsGotId(source_p) ? "" : "~",
-			source_p->username, source_p->sockhost);	
+			source_p->username, source_p->sockhost);
 
 		ServerStats.is_ref++;
 		exit_client(client_p, source_p, &me, "Too many host connections (local)");
@@ -246,7 +249,7 @@ check_client(struct Client *client_p, struct Client *source_p, const char *usern
 				source_p->username, source_p->host,
 				show_ip(NULL, source_p) && !IsIPSpoof(source_p) ? source_p->sockhost : "255.255.255.255");
 
-		ilog(L_FUSER, "Too many connections from %s!%s%s@%s.", 
+		ilog(L_FUSER, "Too many connections from %s!%s%s@%s.",
 			source_p->name, IsGotId(source_p) ? "" : "~",
 			source_p->username, source_p->sockhost);
 
@@ -264,7 +267,7 @@ check_client(struct Client *client_p, struct Client *source_p, const char *usern
 			else
 #endif
 				port = ntohs(((struct sockaddr_in *)&source_p->localClient->listener->addr)->sin_port);
-			
+
 			ServerStats.is_ref++;
 			/* jdc - lists server name & port connections are on */
 			/*       a purely cosmetical change */
@@ -315,27 +318,8 @@ static int
 verify_access(struct Client *client_p, const char *username)
 {
 	struct ConfItem *aconf;
-	char non_ident[USERLEN + 1];
-
-	if(IsGotId(client_p))
-	{
-		aconf = find_address_conf(client_p->host, client_p->sockhost, 
-					client_p->username, client_p->username,
-					(struct sockaddr *) &client_p->localClient->ip,
-					client_p->localClient->ip.ss_family,
-					client_p->localClient->auth_user);
-	}
-	else
-	{
-		rb_strlcpy(non_ident, "~", sizeof(non_ident));
-		rb_strlcat(non_ident, username, sizeof(non_ident));
-		aconf = find_address_conf(client_p->host, client_p->sockhost,
-					non_ident, client_p->username,
-					(struct sockaddr *) &client_p->localClient->ip,
-					client_p->localClient->ip.ss_family,
-					client_p->localClient->auth_user);
-	}
 
+	aconf = find_address_conf_by_client(client_p, username);
 	if(aconf == NULL)
 		return NOT_AUTHORISED;
 
@@ -397,9 +381,40 @@ verify_access(struct Client *client_p, const char *username)
 }
 
 
+/*
+ * find_address_conf_by_client
+ */
+static struct ConfItem *
+find_address_conf_by_client(struct Client *client_p, const char *username)
+{
+	struct ConfItem *aconf;
+	char non_ident[USERLEN + 1];
+
+	if(IsGotId(client_p))
+	{
+		aconf = find_address_conf(client_p->host, client_p->sockhost,
+					client_p->username, client_p->username,
+					(struct sockaddr *) &client_p->localClient->ip,
+					client_p->localClient->ip.ss_family,
+					client_p->localClient->auth_user);
+	}
+	else
+	{
+		rb_strlcpy(non_ident, "~", sizeof(non_ident));
+		rb_strlcat(non_ident, username, sizeof(non_ident));
+		aconf = find_address_conf(client_p->host, client_p->sockhost,
+					non_ident, client_p->username,
+					(struct sockaddr *) &client_p->localClient->ip,
+					client_p->localClient->ip.ss_family,
+					client_p->localClient->auth_user);
+	}
+	return aconf;
+}
+
+
 /*
  * add_ip_limit
- * 
+ *
  * Returns 1 if successful 0 if not
  *
  * This checks if the user has exceed the limits for their class
@@ -570,7 +585,7 @@ detach_conf(struct Client *client_p)
 
 /*
  * attach_conf
- * 
+ *
  * inputs	- client pointer
  * 		- conf pointer
  * output	-
@@ -671,8 +686,7 @@ set_default_conf(void)
 	/* ServerInfo.name is not rehashable */
 	/* ServerInfo.name = ServerInfo.name; */
 	ServerInfo.description = NULL;
-	ServerInfo.network_name = rb_strdup(NETWORK_NAME_DEFAULT);
-	ServerInfo.network_desc = rb_strdup(NETWORK_DESC_DEFAULT);
+	ServerInfo.network_name = NULL;
 
 	memset(&ServerInfo.ip, 0, sizeof(ServerInfo.ip));
 	ServerInfo.specific_ipv4_vhost = 0;
@@ -687,11 +701,12 @@ set_default_conf(void)
 	AdminInfo.email = NULL;
 	AdminInfo.description = NULL;
 
-	ConfigFileEntry.default_operstring = rb_strdup("is an IRC operator");
-	ConfigFileEntry.default_adminstring = rb_strdup("is a Server Administrator");
-	ConfigFileEntry.servicestring = rb_strdup("is a Network Service");
+	ConfigFileEntry.default_operstring = NULL;
+	ConfigFileEntry.default_adminstring = NULL;
+	ConfigFileEntry.servicestring = NULL;
+	ConfigFileEntry.sasl_service = NULL;
 
-	ConfigFileEntry.default_umodes = UMODE_INVISIBLE;	
+	ConfigFileEntry.default_umodes = UMODE_INVISIBLE;
 	ConfigFileEntry.failed_oper_notice = YES;
 	ConfigFileEntry.anti_nick_flood = NO;
 	ConfigFileEntry.disable_fake_channels = NO;
@@ -730,15 +745,14 @@ set_default_conf(void)
 	ConfigFileEntry.fname_operlog = NULL;
 	ConfigFileEntry.fname_foperlog = NULL;
 	ConfigFileEntry.fname_serverlog = NULL;
+	ConfigFileEntry.fname_killlog = NULL;
 	ConfigFileEntry.fname_klinelog = NULL;
 	ConfigFileEntry.fname_operspylog = NULL;
 	ConfigFileEntry.fname_ioerrorlog = NULL;
-	ConfigFileEntry.use_egd = NO;
 	ConfigFileEntry.hide_spoof_ips = YES;
 	ConfigFileEntry.hide_error_messages = 1;
 	ConfigFileEntry.dots_in_ident = 0;
 	ConfigFileEntry.max_targets = MAX_TARGETS_DEFAULT;
-	ConfigFileEntry.egdpool_path = NULL;
 	ConfigFileEntry.use_whois_actually = YES;
 	ConfigFileEntry.burst_away = NO;
 	ConfigFileEntry.collision_fnc = YES;
@@ -778,6 +792,9 @@ set_default_conf(void)
 	ConfigChannel.resv_forcepart = YES;
 	ConfigChannel.channel_target_change = YES;
 	ConfigChannel.disable_local_channels = NO;
+	ConfigChannel.displayed_usercount = 3;
+
+	ConfigChannel.autochanmodes = MODE_TOPICLIMIT | MODE_NOPRIVMSGS;
 
 	ConfigServerHide.flatten_links = 0;
 	ConfigServerHide.links_delay = 300;
@@ -787,11 +804,11 @@ set_default_conf(void)
 	ConfigFileEntry.min_nonwildcard = 4;
 	ConfigFileEntry.min_nonwildcard_simple = 3;
 	ConfigFileEntry.default_floodcount = 8;
-	ConfigFileEntry.default_ident_timeout = 5;
+	ConfigFileEntry.default_ident_timeout = IDENT_TIMEOUT_DEFAULT;
 	ConfigFileEntry.tkline_expire_notices = 0;
 
         ConfigFileEntry.reject_after_count = 5;
-	ConfigFileEntry.reject_ban_time = 300;  
+	ConfigFileEntry.reject_ban_time = 300;
 	ConfigFileEntry.reject_duration = 120;
 	ConfigFileEntry.throttle_count = 4;
 	ConfigFileEntry.throttle_duration = 60;
@@ -805,6 +822,7 @@ set_default_conf(void)
 	ServerInfo.default_max_clients = MAXCONNECTIONS;
 
 	ConfigFileEntry.nicklen = NICKLEN;
+	ConfigFileEntry.certfp_method = RB_SSL_CERTFP_METH_CERT_SHA1;
 
 	if (!alias_dict)
 		alias_dict = irc_dictionary_create(strcasecmp);
@@ -814,15 +832,15 @@ set_default_conf(void)
 #undef NO
 
 /*
- * read_conf() 
+ * read_conf()
  *
  *
- * inputs       - file descriptor pointing to config file to use
+ * inputs       - None
  * output       - None
  * side effects	- Read configuration file.
  */
 static void
-read_conf(FILE * file)
+read_conf(void)
 {
 	lineno = 0;
 
@@ -838,6 +856,9 @@ read_conf(FILE * file)
 static void
 validate_conf(void)
 {
+	if(ConfigFileEntry.default_ident_timeout < 1)
+		ConfigFileEntry.default_ident_timeout = IDENT_TIMEOUT_DEFAULT;
+
 	if(ConfigFileEntry.ts_warn_delta < TS_WARN_DELTA_MIN)
 		ConfigFileEntry.ts_warn_delta = TS_WARN_DELTA_DEFAULT;
 
@@ -847,29 +868,38 @@ validate_conf(void)
 	if(ServerInfo.network_name == NULL)
 		ServerInfo.network_name = rb_strdup(NETWORK_NAME_DEFAULT);
 
-	if(ServerInfo.network_desc == NULL)
-		ServerInfo.network_desc = rb_strdup(NETWORK_DESC_DEFAULT);
-
 	if(ServerInfo.ssld_count < 1)
 		ServerInfo.ssld_count = 1;
 
-	if(!rb_setup_ssl_server(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params))
+	if(!rb_setup_ssl_server(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list))
 	{
 		ilog(L_MAIN, "WARNING: Unable to setup SSL.");
-		ssl_ok = 0;
+		ircd_ssl_ok = 0;
 	} else {
-		ssl_ok = 1;
-		send_new_ssl_certs(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params);
+		ircd_ssl_ok = 1;
+		send_new_ssl_certs(ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list, ConfigFileEntry.certfp_method);
 	}
 
 	if(ServerInfo.ssld_count > get_ssld_count())
 	{
 		int start = ServerInfo.ssld_count - get_ssld_count();
 		/* start up additional ssld if needed */
-		start_ssldaemon(start, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params);
-				
+		start_ssldaemon(start, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list);
 	}
 
+	/* General conf */
+	if (ConfigFileEntry.default_operstring == NULL)
+		ConfigFileEntry.default_operstring = rb_strdup("is an IRC operator");
+
+	if (ConfigFileEntry.default_adminstring == NULL)
+		ConfigFileEntry.default_adminstring = rb_strdup("is a Server Administrator");
+
+	if (ConfigFileEntry.servicestring == NULL)
+		ConfigFileEntry.servicestring = rb_strdup("is a Network Service");
+
+	if (ConfigFileEntry.sasl_service == NULL)
+		ConfigFileEntry.sasl_service = rb_strdup("SaslServ");
+
 	/* RFC 1459 says 1 message per 2 seconds on average and bursts of
 	 * 5 messages are acceptable, so allow at least that.
 	 */
@@ -900,7 +930,7 @@ validate_conf(void)
  *
  * inputs        - pointer to struct ConfItem
  * output        - none
- * Side effects  - links in given struct ConfItem into 
+ * Side effects  - links in given struct ConfItem into
  *                 temporary kline link list
  */
 void
@@ -966,7 +996,7 @@ add_temp_dline(struct ConfItem *aconf)
 }
 
 /* valid_wild_card()
- * 
+ *
  * input        - user buffer, host buffer
  * output       - 0 if invalid, 1 if valid
  * side effects -
@@ -1041,7 +1071,7 @@ find_prop_ban(unsigned int status, const char *user, const char *host)
 }
 
 void
-deactivate_conf(struct ConfItem *aconf, rb_dlink_node *ptr)
+deactivate_conf(struct ConfItem *aconf, rb_dlink_node *ptr, time_t now)
 {
 	int i;
 
@@ -1080,7 +1110,7 @@ deactivate_conf(struct ConfItem *aconf, rb_dlink_node *ptr)
 			del_from_resv_hash(aconf->host, aconf);
 			break;
 	}
-	if (aconf->lifetime != 0 && rb_current_time() < aconf->lifetime)
+	if (aconf->lifetime != 0 && now < aconf->lifetime)
 		aconf->status |= CONF_ILLEGAL;
 	else
 	{
@@ -1116,7 +1146,7 @@ replace_old_ban(struct ConfItem *aconf)
 			aconf->lifetime = aconf->hold;
 		/* Tell deactivate_conf() to destroy it. */
 		oldconf->lifetime = rb_current_time();
-		deactivate_conf(oldconf, ptr);
+		deactivate_conf(oldconf, ptr, oldconf->lifetime);
 	}
 }
 
@@ -1126,13 +1156,15 @@ expire_prop_bans(void *list)
 	rb_dlink_node *ptr;
 	rb_dlink_node *next_ptr;
 	struct ConfItem *aconf;
+	time_t now;
 
+	now = rb_current_time();
 	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, ((rb_dlink_list *) list)->head)
 	{
 		aconf = ptr->data;
 
-		if(aconf->lifetime <= rb_current_time() ||
-				(aconf->hold <= rb_current_time() &&
+		if(aconf->lifetime <= now ||
+				(aconf->hold <= now &&
 				 !(aconf->status & CONF_ILLEGAL)))
 		{
 			/* Alert opers that a TKline expired - Hwy */
@@ -1146,7 +1178,7 @@ expire_prop_bans(void *list)
 						     aconf->host ? aconf->host : "*");
 
 			/* will destroy or mark illegal */
-			deactivate_conf(aconf, ptr);
+			deactivate_conf(aconf, ptr, now);
 		}
 	}
 }
@@ -1196,7 +1228,7 @@ reorganise_temp_kd(void *list)
 
 		if(aconf->hold < (rb_current_time() + (60 * 60)))
 		{
-			rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ? 
+			rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ?
 					&temp_klines[TEMP_MIN] : &temp_dlines[TEMP_MIN]);
 			aconf->port = TEMP_MIN;
 		}
@@ -1204,14 +1236,14 @@ reorganise_temp_kd(void *list)
 		{
 			if(aconf->hold < (rb_current_time() + (1440 * 60)))
 			{
-				rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ? 
+				rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ?
 						&temp_klines[TEMP_HOUR] : &temp_dlines[TEMP_HOUR]);
 				aconf->port = TEMP_HOUR;
 			}
-			else if(aconf->port > TEMP_DAY && 
+			else if(aconf->port > TEMP_DAY &&
 				(aconf->hold < (rb_current_time() + (10080 * 60))))
 			{
-				rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ? 
+				rb_dlinkMoveNode(ptr, list, (aconf->status == CONF_KILL) ?
 						&temp_klines[TEMP_DAY] : &temp_dlines[TEMP_DAY]);
 				aconf->port = TEMP_DAY;
 			}
@@ -1241,7 +1273,7 @@ get_oper_name(struct Client *client_p)
 	}
 
 	rb_snprintf(buffer, sizeof(buffer), "%s!%s@%s{%s}",
-		   client_p->name, client_p->username, 
+		   client_p->name, client_p->username,
 		   client_p->host, client_p->servptr->name);
 	return buffer;
 }
@@ -1251,7 +1283,7 @@ get_oper_name(struct Client *client_p)
  *
  * inputs        - struct ConfItem
  *
- * output         - name 
+ * output         - name
  *                - host
  *                - pass
  *                - user
@@ -1264,7 +1296,7 @@ get_oper_name(struct Client *client_p)
  */
 void
 get_printable_conf(struct ConfItem *aconf, char **name, char **host,
-		   char **pass, char **user, int *port, char **classname)
+		   const char **pass, char **user, int *port, char **classname)
 {
 	static char null[] = "<NULL>";
 	static char zero[] = "default";
@@ -1305,7 +1337,7 @@ get_user_ban_reason(struct ConfItem *aconf)
 }
 
 void
-get_printable_kline(struct Client *source_p, struct ConfItem *aconf, 
+get_printable_kline(struct Client *source_p, struct ConfItem *aconf,
 		    char **host, char **reason,
 		    char **user, char **oper_reason)
 {
@@ -1348,7 +1380,9 @@ read_conf_files(int cold)
 	   FIXME: The full path is in conffilenamebuf first time since we
 	   dont know anything else
 
-	   - Gozem 2002-07-21 
+	   - Gozem 2002-07-21
+
+
 	 */
 	rb_strlcpy(conffilebuf, filename, sizeof(conffilebuf));
 
@@ -1356,7 +1390,15 @@ read_conf_files(int cold)
 	{
 		if(cold)
 		{
+			inotice("Failed in reading configuration file %s, aborting", filename);
 			ilog(L_MAIN, "Failed in reading configuration file %s", filename);
+
+			int e;
+			e = errno;
+
+			inotice("FATAL: %s %s", strerror(e), filename);
+			ilog(L_MAIN, "FATAL: %s %s", strerror(e), filename);
+
 			exit(-1);
 		}
 		else
@@ -1372,7 +1414,10 @@ read_conf_files(int cold)
 		clear_out_old_conf();
 	}
 
-	read_conf(conf_fbfile_in);
+	call_hook(h_conf_read_start, NULL);
+	read_conf();
+	call_hook(h_conf_read_end, NULL);
+
 	fclose(conf_fbfile_in);
 }
 
@@ -1428,8 +1473,6 @@ clear_out_old_conf(void)
 	ServerInfo.description = NULL;
 	rb_free(ServerInfo.network_name);
 	ServerInfo.network_name = NULL;
-	rb_free(ServerInfo.network_desc);
-	ServerInfo.network_desc = NULL;
 
 	ServerInfo.ssld_count = 1;
 
@@ -1450,8 +1493,36 @@ clear_out_old_conf(void)
 	 */
 
 	/* clean out general */
+	rb_free(ConfigFileEntry.default_operstring);
+	ConfigFileEntry.default_operstring = NULL;
+	rb_free(ConfigFileEntry.default_adminstring);
+	ConfigFileEntry.default_adminstring = NULL;
+	rb_free(ConfigFileEntry.servicestring);
+	ConfigFileEntry.servicestring = NULL;
 	rb_free(ConfigFileEntry.kline_reason);
 	ConfigFileEntry.kline_reason = NULL;
+	rb_free(ConfigFileEntry.sasl_service);
+	ConfigFileEntry.sasl_service = NULL;
+
+	/* clean out log */
+	rb_free(ConfigFileEntry.fname_userlog);
+	ConfigFileEntry.fname_userlog = NULL;
+	rb_free(ConfigFileEntry.fname_fuserlog);
+	ConfigFileEntry.fname_fuserlog = NULL;
+	rb_free(ConfigFileEntry.fname_operlog);
+	ConfigFileEntry.fname_operlog = NULL;
+	rb_free(ConfigFileEntry.fname_foperlog);
+	ConfigFileEntry.fname_foperlog = NULL;
+	rb_free(ConfigFileEntry.fname_serverlog);
+	ConfigFileEntry.fname_serverlog = NULL;
+	rb_free(ConfigFileEntry.fname_killlog);
+	ConfigFileEntry.fname_killlog = NULL;
+	rb_free(ConfigFileEntry.fname_klinelog);
+	ConfigFileEntry.fname_klinelog = NULL;
+	rb_free(ConfigFileEntry.fname_operspylog);
+	ConfigFileEntry.fname_operspylog = NULL;
+	rb_free(ConfigFileEntry.fname_ioerrorlog);
+	ConfigFileEntry.fname_ioerrorlog = NULL;
 
 	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, service_list.head)
 	{
@@ -1478,7 +1549,7 @@ clear_out_old_conf(void)
  * conf_add_class_to_conf
  * inputs       - pointer to config item
  * output       - NONE
- * side effects - Add a class pointer to a conf 
+ * side effects - Add a class pointer to a conf
  */
 
 void
@@ -1576,7 +1647,7 @@ yyerror(const char *msg)
 {
 	char newlinebuf[BUFSIZE];
 
-	strip_tabs(newlinebuf, linebuf, strlen(linebuf));
+	strip_tabs(newlinebuf, yy_linebuf, strlen(yy_linebuf));
 
 	ierror("\"%s\", line %d: %s at '%s'", conffilebuf, lineno + 1, msg, newlinebuf);
 	sendto_realops_snomask(SNO_GENERAL, L_ALL, "\"%s\", line %d: %s at '%s'",
