@@ -44,6 +44,8 @@
 #include "newconf.h"
 #include "hash.h"
 #include "irc_dictionary.h"
+#include "s_assert.h"
+#include "logger.h"
 
 rb_dlink_list shared_conf_list;
 rb_dlink_list cluster_conf_list;
@@ -173,7 +175,7 @@ free_remote_conf(struct remote_conf *remote_p)
 }
 
 int
-find_shared_conf(const char *username, const char *host, 
+find_shared_conf(const char *username, const char *host,
 		const char *server, int flags)
 {
 	struct remote_conf *shared_p;
@@ -215,7 +217,7 @@ propagate_generic(struct Client *source_p, const char *command,
 			"ENCAP %s %s %s",
 			target, command, buffer);
 }
-			
+
 void
 cluster_generic(struct Client *source_p, const char *command,
 		int cltype, int cap, const char *format, ...)
@@ -289,7 +291,7 @@ find_oper_conf(const char *username, const char *host, const char *locip, const
 	int bits, cbits;
 	rb_dlink_node *ptr;
 
-	parse_netmask(locip, (struct sockaddr *)&cip, &cbits);
+	parse_netmask(locip, &cip, &cbits);
 
 	RB_DLINK_FOREACH(ptr, oper_conf_list.head)
 	{
@@ -301,7 +303,7 @@ find_oper_conf(const char *username, const char *host, const char *locip, const
 
 		rb_strlcpy(addr, oper_p->host, sizeof(addr));
 
-		if(parse_netmask(addr, (struct sockaddr *)&ip, &bits) != HM_HOST)
+		if(parse_netmask(addr, &ip, &bits) != HM_HOST)
 		{
 			if(ip.ss_family == cip.ss_family &&
 			   comp_with_mask_sock((struct sockaddr *)&ip, (struct sockaddr *)&cip, bits))
@@ -349,6 +351,7 @@ free_server_conf(struct server_conf *server_p)
 	rb_free(server_p->name);
 	rb_free(server_p->host);
 	rb_free(server_p->class_name);
+	rb_free(server_p->certfp);
 	rb_free(server_p);
 }
 
@@ -730,7 +733,7 @@ add_nd_entry(const char *name)
 		return;
 
 	nd = rb_bh_alloc(nd_heap);
-	
+
 	rb_strlcpy(nd->name, name, sizeof(nd->name));
 	nd->expire = rb_current_time() + ConfigFileEntry.nick_delay;
 
