@@ -54,6 +54,7 @@
 #include "blacklist.h"
 #include "substitution.h"
 #include "chmode.h"
+#include "s_assert.h"
 
 static void report_and_set_user_flags(struct Client *, struct ConfItem *);
 void user_welcome(struct Client *source_p);
@@ -147,7 +148,7 @@ show_lusers(struct Client *source_p)
 
 	if((rb_dlink_list_length(&lclient_list) + rb_dlink_list_length(&serv_list)) >
 	   (unsigned long)MaxConnectionCount)
-		MaxConnectionCount = rb_dlink_list_length(&lclient_list) + 
+		MaxConnectionCount = rb_dlink_list_length(&lclient_list) +
 					rb_dlink_list_length(&serv_list);
 
 	sendto_one_numeric(source_p, RPL_LUSERCLIENT, form_str(RPL_LUSERCLIENT),
@@ -156,16 +157,17 @@ show_lusers(struct Client *source_p)
 			   (int)rb_dlink_list_length(&global_serv_list));
 
 	if(rb_dlink_list_length(&oper_list) > 0)
-		sendto_one_numeric(source_p, RPL_LUSEROP, 
-				   form_str(RPL_LUSEROP), rb_dlink_list_length(&oper_list));
+		sendto_one_numeric(source_p, RPL_LUSEROP,
+				   form_str(RPL_LUSEROP),
+				   (int)rb_dlink_list_length(&oper_list));
 
 	if(rb_dlink_list_length(&unknown_list) > 0)
-		sendto_one_numeric(source_p, RPL_LUSERUNKNOWN, 
+		sendto_one_numeric(source_p, RPL_LUSERUNKNOWN,
 				   form_str(RPL_LUSERUNKNOWN),
 				   (int)rb_dlink_list_length(&unknown_list));
 
 	if(rb_dlink_list_length(&global_channel_list) > 0)
-		sendto_one_numeric(source_p, RPL_LUSERCHANNELS, 
+		sendto_one_numeric(source_p, RPL_LUSERCHANNELS,
 				   form_str(RPL_LUSERCHANNELS),
 				   rb_dlink_list_length(&global_channel_list));
 
@@ -173,7 +175,7 @@ show_lusers(struct Client *source_p)
 			   (int)rb_dlink_list_length(&lclient_list),
 			   (int)rb_dlink_list_length(&serv_list));
 
-	sendto_one_numeric(source_p, RPL_LOCALUSERS, 
+	sendto_one_numeric(source_p, RPL_LOCALUSERS,
 			   form_str(RPL_LOCALUSERS),
 			   (int)rb_dlink_list_length(&lclient_list),
 			   Count.max_loc,
@@ -186,7 +188,7 @@ show_lusers(struct Client *source_p)
 
 	sendto_one_numeric(source_p, RPL_STATSCONN,
 			   form_str(RPL_STATSCONN),
-			   MaxConnectionCount, MaxClientCount, 
+			   MaxConnectionCount, MaxClientCount,
 			   Count.totalrestartcount);
 
 	return 0;
@@ -217,7 +219,7 @@ show_lusers(struct Client *source_p)
 */
 
 int
-register_local_user(struct Client *client_p, struct Client *source_p, const char *username)
+register_local_user(struct Client *client_p, struct Client *source_p)
 {
 	struct ConfItem *aconf, *xconf;
 	struct User *user = source_p->user;
@@ -228,7 +230,6 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 
 	s_assert(NULL != source_p);
 	s_assert(MyConnect(source_p));
-	s_assert(source_p->username != username);
 
 	if(source_p == NULL)
 		return -1;
@@ -263,12 +264,12 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 	client_p->localClient->last = rb_current_time();
 
 	/* XXX - fixme. we shouldnt have to build a users buffer twice.. */
-	if(!IsGotId(source_p) && (strchr(username, '[') != NULL))
+	if(!IsGotId(source_p) && (strchr(source_p->username, '[') != NULL))
 	{
 		const char *p;
 		int i = 0;
 
-		p = username;
+		p = source_p->username;
 
 		while(*p && i < USERLEN)
 		{
@@ -278,10 +279,11 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		}
 
 		myusername[i] = '\0';
-		username = myusername;
 	}
+	else
+		rb_strlcpy(myusername, source_p->username, sizeof myusername);
 
-	if((status = check_client(client_p, source_p, username)) < 0)
+	if((status = check_client(client_p, source_p, myusername)) < 0)
 		return (CLIENT_EXITED);
 
 	/* Apply nick override */
@@ -303,7 +305,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 
 		rb_strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
  	}
- 
+
 
 	aconf = source_p->localClient->att_conf;
 
@@ -337,7 +339,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		/* dont replace username if its supposed to be spoofed --fl */
 		if(!IsConfDoSpoofIp(aconf) || !strchr(aconf->info.name, '@'))
 		{
-			p = username;
+			p = myusername;
 
 			if(!IsNoTilde(aconf))
 				source_p->username[i++] = '~';
@@ -373,7 +375,7 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 		else
 			encr = source_p->localClient->passwd;
 
-		if(strcmp(encr, aconf->passwd))
+		if(encr == NULL || strcmp(encr, aconf->passwd))
 		{
 			ServerStats.is_ref++;
 			sendto_one(source_p, form_str(ERR_PASSWDMISMATCH), me.name, source_p->name);
@@ -432,6 +434,14 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 					source_p->sockhost, source_p->preClient->dnsbl_listed->host);
 		else
 		{
+			sendto_realops_snomask(SNO_REJ, L_NETWIDE,
+				"Listed on DNSBL %s: %s (%s@%s) [%s] [%s]",
+				source_p->preClient->dnsbl_listed->host,
+				source_p->name,
+				source_p->username, source_p->host,
+				IsIPSpoof(source_p) ? "255.255.255.255" : source_p->sockhost,
+				source_p->info);
+
 			rb_dlink_list varlist = { NULL, NULL, 0 };
 
 			substitution_append_var(&varlist, "nick", source_p->name);
@@ -582,17 +592,16 @@ register_local_user(struct Client *client_p, struct Client *source_p, const char
 int
 introduce_client(struct Client *client_p, struct Client *source_p, struct User *user, const char *nick, int use_euid)
 {
-	static char ubuf[12];
+	char ubuf[BUFSIZE];
 	struct Client *identifyservice_p;
 	char *p;
 	hook_data_umode_changed hdata;
 	hook_data_client hdata2;
-	char sockhost[HOSTLEN];
 
 	if(MyClient(source_p))
-		send_umode(source_p, source_p, 0, 0, ubuf);
+		send_umode(source_p, source_p, 0, ubuf);
 	else
-		send_umode(NULL, source_p, 0, 0, ubuf);
+		send_umode(NULL, source_p, 0, ubuf);
 
 	if(!*ubuf)
 	{
@@ -602,14 +611,6 @@ introduce_client(struct Client *client_p, struct Client *source_p, struct User *
 
 	s_assert(has_id(source_p));
 
-	if(source_p->sockhost[0] == ':')
-	{
-		sockhost[0] = '0';
-		sockhost[1] = '\0';
-		rb_strlcat(sockhost, source_p->sockhost, sizeof(sockhost));
-	} else
-		strcpy(sockhost, source_p->sockhost);
-		
 	if (use_euid)
 		sendto_server(client_p, NULL, CAP_EUID | CAP_TS6, NOCAPS,
 				":%s EUID %s %d %ld %s %s %s %s %s %s %s :%s",
@@ -617,7 +618,7 @@ introduce_client(struct Client *client_p, struct Client *source_p, struct User *
 				source_p->hopcount + 1,
 				(long) source_p->tsinfo, ubuf,
 				source_p->username, source_p->host,
-				IsIPSpoof(source_p) ? "0" : sockhost,
+				IsIPSpoof(source_p) ? "0" : source_p->sockhost,
 				source_p->id,
 				IsDynSpoof(source_p) ? source_p->orighost : "*",
 				EmptyString(source_p->user->suser) ? "*" : source_p->user->suser,
@@ -629,7 +630,7 @@ introduce_client(struct Client *client_p, struct Client *source_p, struct User *
 		      source_p->hopcount + 1,
 		      (long) source_p->tsinfo, ubuf,
 		      source_p->username, source_p->host,
-		      IsIPSpoof(source_p) ? "0" : sockhost,
+		      IsIPSpoof(source_p) ? "0" : source_p->sockhost,
 		      source_p->id, source_p->info);
 
 	if(!EmptyString(source_p->certfp))
@@ -703,7 +704,7 @@ introduce_client(struct Client *client_p, struct Client *source_p, struct User *
 	return 0;
 }
 
-/* 
+/*
  * valid_hostname - check hostname for validity
  *
  * Inputs       - pointer to user
@@ -724,6 +725,9 @@ valid_hostname(const char *hostname)
 	if(hostname == NULL)
 		return NO;
 
+	if(!strcmp(hostname, "localhost"))
+		return YES;
+
 	if('.' == *p || ':' == *p || '/' == *p)
 		return NO;
 
@@ -750,13 +754,13 @@ valid_hostname(const char *hostname)
 	return YES;
 }
 
-/* 
+/*
  * valid_username - check username for validity
  *
  * Inputs       - pointer to user
  * Output       - YES if valid, NO if not
  * Side effects - NONE
- * 
+ *
  * Absolutely always reject any '*' '!' '?' '@' in an user name
  * reject any odd control characters names.
  * Allow '.' in username to allow for "first.last"
@@ -997,7 +1001,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
 			else
 			{
 				/* Only decrement the oper counts if an oper to begin with
-				 * found by Pat Szuta, Perly , perly@xnet.com 
+				 * found by Pat Szuta, Perly , perly@xnet.com
 				 */
 
 				if(!IsOper(source_p))
@@ -1151,7 +1155,7 @@ user_mode(struct Client *client_p, struct Client *source_p, int parc, const char
  * -avalon
  */
 void
-send_umode(struct Client *client_p, struct Client *source_p, int old, int sendmask, char *umode_buf)
+send_umode(struct Client *client_p, struct Client *source_p, int old, char *umode_buf)
 {
 	int i;
 	int flag;
@@ -1203,7 +1207,7 @@ send_umode(struct Client *client_p, struct Client *source_p, int old, int sendma
  *
  * inputs	-
  * output	- NONE
- * side effects - 
+ * side effects -
  */
 void
 send_umode_out(struct Client *client_p, struct Client *source_p, int old)
@@ -1212,7 +1216,7 @@ send_umode_out(struct Client *client_p, struct Client *source_p, int old)
 	char buf[BUFSIZE];
 	rb_dlink_node *ptr;
 
-	send_umode(NULL, source_p, old, 0, buf);
+	send_umode(NULL, source_p, old, buf);
 
 	RB_DLINK_FOREACH(ptr, serv_list.head)
 	{
@@ -1221,16 +1225,16 @@ send_umode_out(struct Client *client_p, struct Client *source_p, int old)
 		if((target_p != client_p) && (target_p != source_p) && (*buf))
 		{
 			sendto_one(target_p, ":%s MODE %s :%s",
-				   get_id(source_p, target_p), 
+				   get_id(source_p, target_p),
 				   get_id(source_p, target_p), buf);
 		}
 	}
 
 	if(client_p && MyClient(client_p))
-		send_umode(client_p, source_p, old, 0, buf);
+		send_umode(client_p, source_p, old, buf);
 }
 
-/* 
+/*
  * user_welcome
  *
  * inputs	- client pointer to client to welcome
@@ -1255,7 +1259,7 @@ user_welcome(struct Client *source_p)
 		sendto_one_notice(source_p, ":*** Notice -- motd was last changed at %s", user_motd_changed);
 		sendto_one_notice(source_p, ":*** Notice -- Please read the motd if you haven't read it");
 
-		sendto_one(source_p, form_str(RPL_MOTDSTART), 
+		sendto_one(source_p, form_str(RPL_MOTDSTART),
 			   me.name, source_p->name, me.name);
 
 		sendto_one(source_p, form_str(RPL_MOTD),
@@ -1381,17 +1385,14 @@ construct_umodebuf(void)
 			if (user_modes[i] == 0)
 			{
 				orphaned_umodes |= prev_user_modes[i];
-				sendto_realops_snomask(SNO_DEBUG, L_ALL, "Umode +%c is now orphaned", i);
+				user_modes[i] = prev_user_modes[i];
 			}
 			else
-			{
 				orphaned_umodes &= ~prev_user_modes[i];
-				sendto_realops_snomask(SNO_DEBUG, L_ALL, "Orphaned umode +%c is picked up by module", i);
-			}
-			user_modes[i] = prev_user_modes[i];
 		}
 		else
 			prev_user_modes[i] = user_modes[i];
+
 		if (user_modes[i])
 			*ptr++ = (char) i;
 	}
@@ -1413,7 +1414,7 @@ change_nick_user_host(struct Client *target_p,	const char *nick, const char *use
 	va_list ap;
 
 	modeval[0] = '\0';
-	
+
 	if(changed)
 	{
 		target_p->tsinfo = newts;
@@ -1427,7 +1428,7 @@ change_nick_user_host(struct Client *target_p,	const char *nick, const char *use
 		vsnprintf(reason, 255, format, ap);
 		va_end(ap);
 
-		sendto_common_channels_local_butone(target_p, NOCAPS, ":%s!%s@%s QUIT :%s",
+		sendto_common_channels_local_butone(target_p, NOCAPS, CLICAP_CHGHOST, ":%s!%s@%s QUIT :%s",
 				target_p->name, target_p->username, target_p->host,
 				reason);
 
@@ -1452,9 +1453,9 @@ change_nick_user_host(struct Client *target_p,	const char *nick, const char *use
 
 			*mptr = '\0';
 
-			sendto_channel_local_with_capability_butone(target_p, ALL_MEMBERS, NOCAPS, CLICAP_EXTENDED_JOIN, chptr,
+			sendto_channel_local_with_capability_butone(target_p, ALL_MEMBERS, NOCAPS, CLICAP_EXTENDED_JOIN | CLICAP_CHGHOST, chptr,
 								    ":%s!%s@%s JOIN %s", nick, user, host, chptr->chname);
-			sendto_channel_local_with_capability_butone(target_p, ALL_MEMBERS, CLICAP_EXTENDED_JOIN, NOCAPS, chptr,
+			sendto_channel_local_with_capability_butone(target_p, ALL_MEMBERS, CLICAP_EXTENDED_JOIN, CLICAP_CHGHOST, chptr,
 								    ":%s!%s@%s JOIN %s %s :%s", nick, user, host, chptr->chname,
 								    EmptyString(target_p->user->suser) ? "*" : target_p->user->suser,
 								    target_p->info);
@@ -1470,21 +1471,24 @@ change_nick_user_host(struct Client *target_p,	const char *nick, const char *use
 
 		/* Resend away message to away-notify enabled clients. */
 		if (target_p->user->away)
-			sendto_common_channels_local_butone(target_p, CLICAP_AWAY_NOTIFY, ":%s!%s@%s AWAY :%s",
-							    target_p->name, target_p->username, target_p->host,
+			sendto_common_channels_local_butone(target_p, CLICAP_AWAY_NOTIFY, CLICAP_CHGHOST, ":%s!%s@%s AWAY :%s",
+							    nick, user, host,
 							    target_p->user->away);
 
+		sendto_common_channels_local_butone(target_p, CLICAP_CHGHOST, NOCAPS,
+						    ":%s!%s@%s CHGHOST %s %s",
+						    target_p->name, target_p->username, target_p->host, user, host);
+
 		if(MyClient(target_p) && changed_case)
 			sendto_one(target_p, ":%s!%s@%s NICK %s",
-					target_p->name, target_p->username, target_p->host, nick);
+					target_p->name, user, host, nick);
 
 		/* TODO: send some snotes to SNO_NCHANGE/SNO_CCONN/SNO_CCONNEXT? */
 	}
 	else if(changed_case)
 	{
-		sendto_common_channels_local(target_p, NOCAPS, ":%s!%s@%s NICK :%s",
-				target_p->name, target_p->username,
-				target_p->host, nick);
+		sendto_common_channels_local(target_p, NOCAPS, NOCAPS, ":%s!%s@%s NICK :%s",
+				target_p->name, user, host, nick);
 
 		if(MyConnect(target_p))
 			sendto_realops_snomask(SNO_NCHANGE, L_ALL,
