@@ -24,7 +24,6 @@
 #include <ratbox_lib.h>
 #include "stdinc.h"
 
-
 #include "s_conf.h"
 #include "logger.h"
 #include "listener.h"
@@ -37,6 +36,8 @@
 #include "packet.h"
 
 #define ZIPSTATS_TIME           60
+#define MAXPASSFD               4
+#define READSIZE                1024
 
 static void collect_zipstats(void *unused);
 static void ssl_read_ctl(rb_fde_t * F, void *data);
@@ -45,8 +46,6 @@ static int ssld_count;
 static char tmpbuf[READBUF_SIZE];
 static char nul = '\0';
 
-#define MAXPASSFD 4
-#define READSIZE 1024
 typedef struct _ssl_ctl_buf
 {
 	rb_dlink_node node;
@@ -56,7 +55,6 @@ typedef struct _ssl_ctl_buf
 	int nfds;
 } ssl_ctl_buf_t;
 
-
 struct _ssl_ctl
 {
 	rb_dlink_node node;
@@ -66,48 +64,33 @@ struct _ssl_ctl
 	pid_t pid;
 	rb_dlink_list readq;
 	rb_dlink_list writeq;
+	uint8_t shutdown;
 	uint8_t dead;
 };
 
 static void send_new_ssl_certs_one(ssl_ctl_t * ctl, const char *ssl_cert,
-				   const char *ssl_private_key, const char *ssl_dh_params);
+				   const char *ssl_private_key, const char *ssl_dh_params,
+				   const char *ssl_cipher_list);
 static void send_init_prng(ssl_ctl_t * ctl, prng_seed_t seedtype, const char *path);
-
+static void send_certfp_method(ssl_ctl_t *ctl, int method);
 
 static rb_dlink_list ssl_daemons;
 
-static inline int32_t
-buf_to_int32(char *buf)
+static inline uint32_t
+buf_to_uint32(char *buf)
 {
-	int32_t x;
+	uint32_t x;
 	memcpy(&x, buf, sizeof(x));
 	return x;
 }
 
 static inline void
-int32_to_buf(char *buf, int32_t x)
+uint32_to_buf(char *buf, uint32_t x)
 {
 	memcpy(buf, &x, sizeof(x));
 	return;
 }
 
-
-static inline uint16_t
-buf_to_uint16(char *buf)
-{
-	uint16_t x;
-	memcpy(&x, buf, sizeof(x));
-	return x;
-}
-
-static inline void
-uint16_to_buf(char *buf, uint16_t x)
-{
-	memcpy(buf, &x, sizeof(x));
-	return;
-}
-
-
 static ssl_ctl_t *
 allocate_ssl_daemon(rb_fde_t * F, rb_fde_t * P, int pid)
 {
@@ -164,6 +147,30 @@ static int ssld_spin_count = 0;
 static time_t last_spin;
 static int ssld_wait = 0;
 
+void
+restart_ssld(void)
+{
+	rb_dlink_node *ptr, *next;
+	ssl_ctl_t *ctl;
+
+	RB_DLINK_FOREACH_SAFE(ptr, next, ssl_daemons.head)
+	{
+		ctl = ptr->data;
+		if(ctl->dead)
+			continue;
+		if(ctl->shutdown)
+			continue;
+		ctl->shutdown = 1;
+		ssld_count--;
+		if(!ctl->cli_count)
+		{
+			rb_kill(ctl->pid, SIGKILL);
+			free_ssl_daemon(ctl);
+		}
+	}
+
+	start_ssldaemon(ServerInfo.ssld_count, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list);
+}
 
 static void
 ssl_killall(void)
@@ -176,8 +183,11 @@ ssl_killall(void)
 		if(ctl->dead)
 			continue;
 		ctl->dead = 1;
-		ssld_count--;
+		if(!ctl->shutdown)
+			ssld_count--;
 		rb_kill(ctl->pid, SIGKILL);
+		if(!ctl->cli_count)
+			free_ssl_daemon(ctl);
 	}
 }
 
@@ -188,11 +198,15 @@ ssl_dead(ssl_ctl_t * ctl)
 		return;
 
 	ctl->dead = 1;
-	ssld_count--;
 	rb_kill(ctl->pid, SIGKILL);	/* make sure the process is really gone */
-	ilog(L_MAIN, "ssld helper died - attempting to restart");
-	sendto_realops_snomask(SNO_GENERAL, L_ALL, "ssld helper died - attempting to restart");
-	start_ssldaemon(1, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params);
+
+	if(!ctl->shutdown)
+	{
+		ssld_count--;
+		ilog(L_MAIN, "ssld helper died - attempting to restart");
+		sendto_realops_snomask(SNO_GENERAL, L_ALL, "ssld helper died - attempting to restart");
+		start_ssldaemon(1, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list);
+	}
 }
 
 static void
@@ -220,12 +234,12 @@ restart_ssld_event(void *unused)
 		int start = ServerInfo.ssld_count - get_ssld_count();
 		ilog(L_MAIN, "Attempting to restart ssld processes");
 		sendto_realops_snomask(SNO_GENERAL, L_ALL, "Attempt to restart ssld processes");
-		start_ssldaemon(start, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params);
+		start_ssldaemon(start, ServerInfo.ssl_cert, ServerInfo.ssl_private_key, ServerInfo.ssl_dh_params, ServerInfo.ssl_cipher_list);
 	}
 }
 
 int
-start_ssldaemon(int count, const char *ssl_cert, const char *ssl_private_key, const char *ssl_dh_params)
+start_ssldaemon(int count, const char *ssl_cert, const char *ssl_private_key, const char *ssl_dh_params, const char *ssl_cipher_list)
 {
 	rb_fde_t *F1, *F2;
 	rb_fde_t *P1, *P2;
@@ -289,7 +303,7 @@ start_ssldaemon(int count, const char *ssl_cert, const char *ssl_private_key, co
 			ilog(L_MAIN, "Unable to create ssld - rb_socketpair failed: %s", strerror(errno));
 			return started;
 		}
-		
+
 		rb_set_buffers(F1, READBUF_SIZE);
 		rb_set_buffers(F2, READBUF_SIZE);
 		rb_snprintf(fdarg, sizeof(fdarg), "%d", rb_get_fd(F2));
@@ -322,16 +336,14 @@ start_ssldaemon(int count, const char *ssl_cert, const char *ssl_private_key, co
 		rb_close(F2);
 		rb_close(P1);
 		ctl = allocate_ssl_daemon(F1, P2, pid);
-		if(ssl_ok)
+		if(ircd_ssl_ok)
 		{
-			if(ConfigFileEntry.use_egd && (ConfigFileEntry.egdpool_path != NULL))
-				send_init_prng(ctl, RB_PRNG_EGD, ConfigFileEntry.egdpool_path);
-			else
-				send_init_prng(ctl, RB_PRNG_DEFAULT, NULL);
+			send_init_prng(ctl, RB_PRNG_DEFAULT, NULL);
+			send_certfp_method(ctl, ConfigFileEntry.certfp_method);
+
+			if(ssl_cert != NULL)
+				send_new_ssl_certs_one(ctl, ssl_cert, ssl_private_key, ssl_dh_params, ssl_cipher_list);
 		}
-		if(ssl_ok && ssl_cert != NULL && ssl_private_key != NULL)
-			send_new_ssl_certs_one(ctl, ssl_cert, ssl_private_key,
-					       ssl_dh_params != NULL ? ssl_dh_params : "");
 		ssl_read_ctl(ctl->F, ctl);
 		ssl_do_pipe(P2, ctl);
 
@@ -344,9 +356,8 @@ ssl_process_zipstats(ssl_ctl_t * ctl, ssl_ctl_buf_t * ctl_buf)
 {
 	struct Client *server;
 	struct ZipStats *zips;
-	int parc;
 	char *parv[7];
-	parc = rb_string_to_array(ctl_buf->buf, parv, 6);
+	(void) rb_string_to_array(ctl_buf->buf, parv, 6);
 	server = find_server(NULL, parv[1]);
 	if(server == NULL || server->localClient == NULL || !IsCapable(server, CAP_ZIP))
 		return;
@@ -376,14 +387,14 @@ ssl_process_dead_fd(ssl_ctl_t * ctl, ssl_ctl_buf_t * ctl_buf)
 {
 	struct Client *client_p;
 	char reason[256];
-	int32_t fd;
+	uint32_t fd;
 
 	if(ctl_buf->buflen < 6)
 		return;		/* bogus message..drop it.. XXX should warn here */
 
-	fd = buf_to_int32(&ctl_buf->buf[1]);
+	fd = buf_to_uint32(&ctl_buf->buf[1]);
 	rb_strlcpy(reason, &ctl_buf->buf[5], sizeof(reason));
-	client_p = find_cli_fd_hash(fd);
+	client_p = find_cli_connid_hash(fd);
 	if(client_p == NULL)
 		return;
 	if(IsAnyServer(client_p) || IsRegistered(client_p))
@@ -402,27 +413,79 @@ ssl_process_dead_fd(ssl_ctl_t * ctl, ssl_ctl_buf_t * ctl_buf)
 	exit_client(client_p, client_p, &me, reason);
 }
 
+static void
+ssl_process_cipher_string(ssl_ctl_t *ctl, ssl_ctl_buf_t *ctl_buf)
+{
+	struct Client *client_p;
+	const char *cstring;
+	uint32_t fd;
+
+	if(ctl_buf->buflen < 6)
+		return;         /* bogus message..drop it.. XXX should warn here */
+
+	fd = buf_to_uint32(&ctl_buf->buf[1]);
+	cstring = (const char *)&ctl_buf->buf[5];
+
+	if(EmptyString(cstring))
+		return;
+
+	client_p = find_cli_connid_hash(fd);
+	if(client_p != NULL && client_p->localClient != NULL)
+	{
+		rb_free(client_p->localClient->cipher_string);
+		client_p->localClient->cipher_string = rb_strdup(cstring);
+	}
+}
+
 static void
 ssl_process_certfp(ssl_ctl_t * ctl, ssl_ctl_buf_t * ctl_buf)
 {
 	struct Client *client_p;
-	int32_t fd;
+	uint32_t fd;
+	uint32_t certfp_method;
+	uint32_t len;
 	uint8_t *certfp;
 	char *certfp_string;
-	int i;
+	const char *method_string;
+	int method_len;
 
-	if(ctl_buf->buflen != 5 + RB_SSL_CERTFP_LEN)
+	if(ctl_buf->buflen > 13 + RB_SSL_CERTFP_LEN)
 		return;		/* bogus message..drop it.. XXX should warn here */
 
-	fd = buf_to_int32(&ctl_buf->buf[1]);
-	certfp = (uint8_t *)&ctl_buf->buf[5];
-	client_p = find_cli_fd_hash(fd);
+	fd = buf_to_uint32(&ctl_buf->buf[1]);
+	certfp_method = buf_to_uint32(&ctl_buf->buf[5]);
+	len = buf_to_uint32(&ctl_buf->buf[9]);
+	certfp = (uint8_t *)&ctl_buf->buf[13];
+	client_p = find_cli_connid_hash(fd);
 	if(client_p == NULL)
 		return;
+
+	switch (certfp_method) {
+	case RB_SSL_CERTFP_METH_CERT_SHA1:
+		method_string = CERTFP_PREFIX_CERT_SHA1;
+		break;
+	case RB_SSL_CERTFP_METH_CERT_SHA256:
+		method_string = CERTFP_PREFIX_CERT_SHA256;
+		break;
+	case RB_SSL_CERTFP_METH_CERT_SHA512:
+		method_string = CERTFP_PREFIX_CERT_SHA512;
+		break;
+	case RB_SSL_CERTFP_METH_SPKI_SHA256:
+		method_string = CERTFP_PREFIX_SPKI_SHA256;
+		break;
+	case RB_SSL_CERTFP_METH_SPKI_SHA512:
+		method_string = CERTFP_PREFIX_SPKI_SHA512;
+		break;
+	default:
+		return;
+	}
+	method_len = strlen(method_string);
+
 	rb_free(client_p->certfp);
-	certfp_string = rb_malloc(RB_SSL_CERTFP_LEN * 2 + 1);
-	for(i = 0; i < RB_SSL_CERTFP_LEN; i++)
-		rb_snprintf(certfp_string + 2 * i, 3, "%02x",
+	certfp_string = rb_malloc(method_len + len * 2 + 1);
+	rb_strlcpy(certfp_string, method_string, method_len + len * 2 + 1);
+	for(uint32_t i = 0; i < len; i++)
+		rb_snprintf(certfp_string + method_len + 2 * i, 3, "%02x",
 				certfp[i]);
 	client_p->certfp = certfp_string;
 }
@@ -442,11 +505,14 @@ ssl_process_cmd_recv(ssl_ctl_t * ctl)
 		switch (*ctl_buf->buf)
 		{
 		case 'N':
-			ssl_ok = 0;	/* ssld says it can't do ssl/tls */
+			ircd_ssl_ok = 0;	/* ssld says it can't do ssl/tls */
 			break;
 		case 'D':
 			ssl_process_dead_fd(ctl, ctl_buf);
 			break;
+		case 'C':
+			ssl_process_cipher_string(ctl, ctl_buf);
+			break;
 		case 'F':
 			ssl_process_certfp(ctl, ctl_buf);
 			break;
@@ -454,18 +520,19 @@ ssl_process_cmd_recv(ssl_ctl_t * ctl)
 			ssl_process_zipstats(ctl, ctl_buf);
 			break;
 		case 'I':
-			ssl_ok = 0;
+			ircd_ssl_ok = 0;
 			ilog(L_MAIN, "%s", cannot_setup_ssl);
 			sendto_realops_snomask(SNO_GENERAL, L_ALL, "%s", cannot_setup_ssl);
+			break;
 		case 'U':
-			zlib_ok = 0;
-			ssl_ok = 0;
+			ircd_zlib_ok = 0;
+			ircd_ssl_ok = 0;
 			ilog(L_MAIN, "%s", no_ssl_or_zlib);
 			sendto_realops_snomask(SNO_GENERAL, L_ALL, "%s", no_ssl_or_zlib);
 			ssl_killall();
 			break;
 		case 'z':
-			zlib_ok = 0;
+			ircd_zlib_ok = 0;
 			break;
 		default:
 			ilog(L_MAIN, "Received invalid command from ssld: %s", ctl_buf->buf);
@@ -479,7 +546,6 @@ ssl_process_cmd_recv(ssl_ctl_t * ctl)
 
 }
 
-
 static void
 ssl_read_ctl(rb_fde_t * F, void *data)
 {
@@ -525,6 +591,8 @@ which_ssld(void)
 		ctl = ptr->data;
 		if(ctl->dead)
 			continue;
+		if(ctl->shutdown)
+			continue;
 		if(lowest == NULL)
 		{
 			lowest = ctl;
@@ -597,13 +665,22 @@ ssl_cmd_write_queue(ssl_ctl_t * ctl, rb_fde_t ** F, int count, const void *buf,
 	ssl_write_ctl(ctl->F, ctl);
 }
 
-
 static void
-send_new_ssl_certs_one(ssl_ctl_t * ctl, const char *ssl_cert, const char *ssl_private_key, const char *ssl_dh_params)
+send_new_ssl_certs_one(ssl_ctl_t * ctl, const char *ssl_cert, const char *ssl_private_key,
+                       const char *ssl_dh_params, const char *ssl_cipher_list)
 {
 	size_t len;
 
-	len = strlen(ssl_cert) + strlen(ssl_private_key) + strlen(ssl_dh_params) + 5;
+	if (ssl_private_key == NULL)
+		ssl_private_key = ssl_cert;
+
+	if (ssl_dh_params == NULL)
+		ssl_dh_params = "";
+
+	if (ssl_cipher_list == NULL)
+		ssl_cipher_list = "";
+
+	len = strlen(ssl_cert) + strlen(ssl_private_key) + strlen(ssl_dh_params) + strlen(ssl_cipher_list) + 6;
 	if(len > sizeof(tmpbuf))
 	{
 		sendto_realops_snomask(SNO_GENERAL, L_ALL,
@@ -614,8 +691,8 @@ send_new_ssl_certs_one(ssl_ctl_t * ctl, const char *ssl_cert, const char *ssl_pr
 		     len, sizeof(tmpbuf));
 		return;
 	}
-	len = rb_snprintf(tmpbuf, sizeof(tmpbuf), "K%c%s%c%s%c%s%c", nul, ssl_cert, nul,
-			  ssl_private_key, nul, ssl_dh_params, nul);
+	len = rb_snprintf(tmpbuf, sizeof(tmpbuf), "K%c%s%c%s%c%s%c%s%c", nul, ssl_cert,
+	                  nul, ssl_private_key, nul, ssl_dh_params, nul, ssl_cipher_list, nul);
 	ssl_cmd_write_queue(ctl, NULL, 0, tmpbuf, len);
 }
 
@@ -647,25 +724,39 @@ send_init_prng(ssl_ctl_t * ctl, prng_seed_t seedtype, const char *path)
 	ssl_cmd_write_queue(ctl, NULL, 0, tmpbuf, len);
 }
 
+static void
+send_certfp_method(ssl_ctl_t *ctl, int method)
+{
+	char buf[5];
+
+	buf[0] = 'F';
+	uint32_to_buf(&buf[1], method);
+	ssl_cmd_write_queue(ctl, NULL, 0, buf, sizeof(buf));
+}
+
 void
-send_new_ssl_certs(const char *ssl_cert, const char *ssl_private_key, const char *ssl_dh_params)
+send_new_ssl_certs(const char *ssl_cert, const char *ssl_private_key, const char *ssl_dh_params, const char *ssl_cipher_list, const int method)
 {
 	rb_dlink_node *ptr;
-	if(ssl_cert == NULL || ssl_private_key == NULL || ssl_dh_params == NULL)
+	if(ssl_cert == NULL)
 	{
-		ssl_ok = 0;
+		ircd_ssl_ok = 0;
 		return;
 	}
 	RB_DLINK_FOREACH(ptr, ssl_daemons.head)
 	{
 		ssl_ctl_t *ctl = ptr->data;
-		send_new_ssl_certs_one(ctl, ssl_cert, ssl_private_key, ssl_dh_params);
+
+		if (ctl->dead || ctl->shutdown)
+			continue;
+
+		send_certfp_method(ctl, method);
+		send_new_ssl_certs_one(ctl, ssl_cert, ssl_private_key, ssl_dh_params, ssl_cipher_list);
 	}
 }
 
-
 ssl_ctl_t *
-start_ssld_accept(rb_fde_t * sslF, rb_fde_t * plainF, int32_t id)
+start_ssld_accept(rb_fde_t * sslF, rb_fde_t * plainF, uint32_t id)
 {
 	rb_fde_t *F[2];
 	ssl_ctl_t *ctl;
@@ -674,7 +765,7 @@ start_ssld_accept(rb_fde_t * sslF, rb_fde_t * plainF, int32_t id)
 	F[1] = plainF;
 
 	buf[0] = 'A';
-	int32_to_buf(&buf[1], id);
+	uint32_to_buf(&buf[1], id);
 	ctl = which_ssld();
 	ctl->cli_count++;
 	ssl_cmd_write_queue(ctl, F, 2, buf, sizeof(buf));
@@ -682,7 +773,7 @@ start_ssld_accept(rb_fde_t * sslF, rb_fde_t * plainF, int32_t id)
 }
 
 ssl_ctl_t *
-start_ssld_connect(rb_fde_t * sslF, rb_fde_t * plainF, int32_t id)
+start_ssld_connect(rb_fde_t * sslF, rb_fde_t * plainF, uint32_t id)
 {
 	rb_fde_t *F[2];
 	ssl_ctl_t *ctl;
@@ -691,7 +782,7 @@ start_ssld_connect(rb_fde_t * sslF, rb_fde_t * plainF, int32_t id)
 	F[1] = plainF;
 
 	buf[0] = 'C';
-	int32_to_buf(&buf[1], id);
+	uint32_to_buf(&buf[1], id);
 
 	ctl = which_ssld();
 	ctl->cli_count++;
@@ -706,16 +797,21 @@ ssld_decrement_clicount(ssl_ctl_t * ctl)
 		return;
 
 	ctl->cli_count--;
+	if(ctl->shutdown && !ctl->cli_count)
+	{
+		ctl->dead = 1;
+		rb_kill(ctl->pid, SIGKILL);
+	}
 	if(ctl->dead && !ctl->cli_count)
 	{
 		free_ssl_daemon(ctl);
 	}
 }
 
-/* 
+/*
  * what we end up sending to the ssld process for ziplinks is the following
- * Z[ourfd][level][RECVQ]  
- * Z = ziplinks command	= buf[0]   
+ * Z[ourfd][level][RECVQ]
+ * Z = ziplinks command	= buf[0]
  * ourfd = Our end of the socketpair = buf[1..4]
  * level = zip level buf[5]
  * recvqlen = our recvq len = buf[6-7]
@@ -732,10 +828,9 @@ start_zlib_session(void *data)
 	rb_fde_t *F[2];
 	rb_fde_t *xF1, *xF2;
 	char *buf;
-	char buf2[9];
 	void *recvq_start;
 
-	size_t hdr = (sizeof(uint8_t) * 2) + sizeof(int32_t);
+	size_t hdr = (sizeof(uint8_t) * 2) + sizeof(uint32_t);
 	size_t len;
 	int cpylen, left;
 
@@ -758,7 +853,7 @@ start_zlib_session(void *data)
 	buf = rb_malloc(len);
 	level = ConfigFileEntry.compression_level;
 
-	int32_to_buf(&buf[1], rb_get_fd(server->localClient->F));
+	uint32_to_buf(&buf[1], server->localClient->zconnid);
 	buf[5] = (char) level;
 
 	recvq_start = &buf[6];
@@ -784,24 +879,10 @@ start_zlib_session(void *data)
 		exit_client(server, server, server, "Error creating zlib socketpair");
 		return;
 	}
-	
-	if(IsSSL(server))
-	{
-		/* tell ssld the new connid for the ssl part*/
-		buf2[0] = 'Y';
-		int32_to_buf(&buf2[1], rb_get_fd(server->localClient->F));
-		int32_to_buf(&buf2[5], rb_get_fd(xF2));
-		ssl_cmd_write_queue(server->localClient->ssl_ctl, NULL, 0, buf2, sizeof(buf2));
-	}
-
 
 	F[0] = server->localClient->F;
 	F[1] = xF1;
-	del_from_cli_fd_hash(server);
 	server->localClient->F = xF2;
-	/* need to redo as what we did before isn't valid now */
-	int32_to_buf(&buf[1], rb_get_fd(server->localClient->F));
-	add_to_cli_fd_hash(server);
 
 	server->localClient->z_ctl = which_ssld();
 	server->localClient->z_ctl->cli_count++;
@@ -814,13 +895,13 @@ collect_zipstats(void *unused)
 {
 	rb_dlink_node *ptr;
 	struct Client *target_p;
-	char buf[sizeof(uint8_t) + sizeof(int32_t) + HOSTLEN];
+	char buf[sizeof(uint8_t) + sizeof(uint32_t) + HOSTLEN];
 	void *odata;
 	size_t len;
-	int32_t id;
+	uint32_t id;
 
 	buf[0] = 'S';
-	odata = buf + sizeof(uint8_t) + sizeof(int32_t);
+	odata = buf + sizeof(uint8_t) + sizeof(uint32_t);
 
 	RB_DLINK_FOREACH(ptr, serv_list.head)
 	{
@@ -830,7 +911,7 @@ collect_zipstats(void *unused)
 			len = sizeof(uint8_t) + sizeof(uint32_t);
 
 			id = rb_get_fd(target_p->localClient->F);
-			int32_to_buf(&buf[1], rb_get_fd(target_p->localClient->F));
+			uint32_to_buf(&buf[1], id);
 			rb_strlcpy(odata, target_p->name, (sizeof(buf) - len));
 			len += strlen(odata) + 1;	/* Get the \0 as well */
 			ssl_cmd_write_queue(target_p->localClient->z_ctl, NULL, 0, buf, len);
@@ -859,9 +940,23 @@ get_ssld_count(void)
 	return ssld_count;
 }
 
+void
+ssld_foreach_info(void (*func)(void *data, pid_t pid, int cli_count, enum ssld_status status), void *data)
+{
+	rb_dlink_node *ptr, *next;
+	ssl_ctl_t *ctl;
+	RB_DLINK_FOREACH_SAFE(ptr, next, ssl_daemons.head)
+	{
+		ctl = ptr->data;
+		func(data, ctl->pid, ctl->cli_count,
+			ctl->dead ? SSLD_DEAD :
+				(ctl->shutdown ? SSLD_SHUTDOWN : SSLD_ACTIVE));
+	}
+}
+
 void
 init_ssld(void)
 {
 	rb_event_addish("collect_zipstats", collect_zipstats, NULL, ZIPSTATS_TIME);
-	rb_event_addish("cleanup_dead_ssld", cleanup_dead_ssl, NULL, 1200);
+	rb_event_addish("cleanup_dead_ssld", cleanup_dead_ssl, NULL, 60);
 }
