@@ -78,8 +78,10 @@
 #include "numeric.h"
 #include "ircd.h"
 #include "s_conf.h"
+#include "s_user.h"
 #include "supported.h"
 #include "chmode.h"
+#include "send.h"
 
 rb_dlink_list isupportlist;
 
@@ -220,7 +222,17 @@ isupport_string(const void *ptr)
 const char *
 isupport_stringptr(const void *ptr)
 {
-	return *(char * const *)ptr;	
+	return *(char * const *)ptr;
+}
+
+static const char *
+isupport_umode(const void *ptr)
+{
+	const char *str;
+
+	str = ptr;
+	return ConfigFileEntry.oper_only_umodes &
+		user_modes[(unsigned char)*str] ? NULL : str;
 }
 
 static const char *
@@ -314,23 +326,15 @@ init_isupport(void)
 	add_isupport("MAXLIST", isupport_maxlist, NULL);
 	add_isupport("MODES", isupport_intptr, &maxmodes);
 	add_isupport("NETWORK", isupport_stringptr, &ServerInfo.network_name);
-	add_isupport("KNOCK", isupport_boolean, &ConfigChannel.use_knock);
 	add_isupport("STATUSMSG", isupport_string, "@+");
-	add_isupport("CALLERID", isupport_string, "g");
+	add_isupport("CALLERID", isupport_umode, "g");
 	add_isupport("CASEMAPPING", isupport_string, "rfc1459");
-	add_isupport("CHARSET", isupport_string, "ascii");
 	add_isupport("NICKLEN", isupport_nicklen, NULL);
 	add_isupport("MAXNICKLEN", isupport_intptr, &maxnicklen);
 	add_isupport("CHANNELLEN", isupport_intptr, &channellen);
 	add_isupport("TOPICLEN", isupport_intptr, &topiclen);
-	add_isupport("ETRACE", isupport_string, "");
-	add_isupport("CPRIVMSG", isupport_string, "");
-	add_isupport("CNOTICE", isupport_string, "");
-	add_isupport("DEAF", isupport_string, "D");
-	add_isupport("MONITOR", isupport_intptr, &ConfigFileEntry.max_monitor);
-	add_isupport("FNC", isupport_string, "");
+	add_isupport("DEAF", isupport_umode, "D");
 	add_isupport("TARGMAX", isupport_targmax, NULL);
 	add_isupport("EXTBAN", isupport_extban, NULL);
-	add_isupport("WHOX", isupport_string, "");
 	add_isupport("CLIENTVER", isupport_string, "3.0");
 }
