@@ -5,8 +5,8 @@
 ** md5 patch by W. Campbell <wcampbel@botbay.net>
 ** Modernization, getopt, etc for the Hybrid IRCD team
 ** by W. Campbell
-** 
-** /dev/random for salt generation added by 
+**
+** /dev/random for salt generation added by
 ** Aaron Sethman <androsyn@ratbox.org>
 **
 ** $Id: mkpasswd.c 26439 2009-02-01 15:27:24Z jilles $
@@ -55,9 +55,6 @@ static void brief_usage(void);
 static char saltChars[] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
        /* 0 .. 63, ascii - 64 */
 
-extern char *optarg;
-
-
 #ifdef __MINGW32__
 #include <conio.h>
 #ifdef PASS_MAX
@@ -72,7 +69,7 @@ getpass(const char *prompt)
 	int c;
 	int i = 0;
 
-	memset(getpassbuf, sizeof(getpassbuf), 0);
+	memset(getpassbuf, 0, sizeof(getpassbuf));
 	fputs(prompt, stderr);
 	for(;;)
 	{
@@ -101,6 +98,7 @@ main(int argc, char *argv[])
 	int c;
 	char *saltpara = NULL;
 	char *salt;
+	char *hashed, *hashed2;
 	int flag = 0;
 	int length = 0;		/* Not Set */
 	int rounds = 0;		/* Not set, since extended DES needs 25 and blowfish needs
@@ -242,14 +240,40 @@ main(int argc, char *argv[])
 	if(flag & FLAG_PASS)
 	{
 		if(!plaintext)
-			printf("Please enter a valid password\n");
+		{
+			fprintf(stderr, "Please enter a valid password\n");
+			return 1;
+		}
+
+		hashed = rb_crypt(plaintext, salt);
 	}
 	else
 	{
 		plaintext = getpass("plaintext: ");
+		hashed = rb_crypt(plaintext, salt);
+		if (!hashed)
+		{
+			fprintf(stderr, "rb_crypt() failed\n");
+			return 1;
+		}
+		hashed = strdup(hashed);
+
+		plaintext = getpass("again: ");
+		hashed2 = rb_crypt(plaintext, salt);
+		if (!hashed2)
+		{
+			fprintf(stderr, "rb_crypt() failed\n");
+			return 1;
+		}
+
+		if (strcmp(hashed, hashed2) != 0)
+		{
+			fprintf(stderr, "Passwords do not match\n");
+			return 1;
+		}
 	}
 
-	printf("%s\n", rb_crypt(plaintext, salt));
+	printf("%s\n", hashed);
 	return 0;
 }
 
